/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_ERROR_H
#define DI_ERROR_H

#include "di_error_enums.h"

#include <stdio.h>

extern void DI_failure(DI_ErrorLevel level,
		       DI_ErrorCode  code, 
		       const char *c1, const char *c2 = 0, const char *c3 = 0);

extern void DI_warning(DI_ErrorCode  code, const char *c1, const char *c2 = 0);

extern void DI_error(DI_ErrorCode  code, const char *c1, const char *c2 = 0);
extern void DI_error(DI_ErrorCode  code, 
		     const char *c1, 
		     const char *c2,
		     const char *c3);
extern void DI_error2(DI_ErrorCode  code, const char *msg, ...);

extern void DI_fatal(DI_ErrorCode  code, const char *c1, const char *c2 = 0);
extern void DI_fatal2(DI_ErrorCode  code, const char *msg, ...);

extern void DI_trace(DI_ErrorCode  code, const char *c1, const char *c2 = 0);
extern void DI_trace(const char *c1, const char *c2 = 0);
extern void DI_trace(const char *c1, const char *c2, int i);
extern void DI_trace(const char *c1, int i);
extern void DI_trace2(const char *msg, ...);


extern FILE *DI_getErrorStream(DI_ErrorLevel level);

extern FILE *DI_traceFile;

extern bool     DI_errorsToFile;

/** Set this function pointer if you want to hook the error messages
    to some other source (in addition to normal output). */

extern void (*DI_errorHandle) (DI_ErrorLevel, const char *);

#endif  // DI_ERROR_H
