/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_TIME_H
#define DI_TIME_H

#include <math.h>

#ifndef WIN32
#include <sys/time.h>
#else
#include <winsock.h>
#include <di_types.h>
#endif

/** Time measurement. */
class DI_Time
{
public:
  DI_Time() {}
  ~DI_Time() {}

  /// Gets the current time.
  void getTimeOfDay() { 
#ifndef WIN32
	  gettimeofday(&m_time, 0); 
#else
	int temp=timeGetTime();
	m_time.tv_sec=temp/1000;
	m_time.tv_usec=temp*1000;
#endif
  }

  /// Compare two times (returns seconds)
  double secsTo(const DI_Time &t) const
  { return (t.m_time.tv_sec - m_time.tv_sec) + 
      ((int)(t.m_time.tv_usec - m_time.tv_usec)) * 0.000001; }

  /// Compare two times (returns microseconds)
  double usecsTo(const DI_Time &t) const
  { return (t.m_time.tv_sec - m_time.tv_sec) * 1000000.0 + 
      (int) (t.m_time.tv_usec - m_time.tv_usec); }

  double secsSince() 
  { DI_Time tmp; tmp.getTimeOfDay(); return secsTo(tmp); }

  void setSecs(float seconds) 
  { 
    m_time.tv_sec = (time_t) floor(seconds); 
    m_time.tv_usec = (long) 
      ((unsigned) ((seconds - (float) m_time.tv_sec) * 1000000.0f)) % 1000000;
  }
  
  void add(const DI_Time & that)
  {
    m_time.tv_sec += that.m_time.tv_sec;
    m_time.tv_usec += that.m_time.tv_usec;
    if(m_time.tv_usec > 1000000) {
      m_time.tv_usec -= 1000000;
      m_time.tv_sec += 1;
    }
  }
  
  inline bool operator < (const DI_Time & that)
  { 
    if(m_time.tv_sec  < that.m_time.tv_sec)  return true;
    if(m_time.tv_sec == that.m_time.tv_sec &&
       m_time.tv_usec < that.m_time.tv_usec) return true;
    return false;
  }

protected:
  struct timeval m_time;
};


#endif

