/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_IMAGE_H
#define DI_IMAGE_H


enum DI_ImageFormat 
{
  DI_IMAGE_UNKNOWN,

  DI_IMAGE_GRAYSCALE,

  DI_IMAGE_YUV_411 = 10,
  DI_IMAGE_YUV_420,
  DI_IMAGE_YUV_420P,
  DI_IMAGE_YUV_422,

  DI_IMAGE_RGB = 128,
  DI_IMAGE_RGBA
};

enum DI_PlaneType 
{
  DI_PLANE_UNKNOWN,

  DI_PLANE_GRAYSCALE,

  DI_PLANE_RED,
  DI_PLANE_GREEN,
  DI_PLANE_BLUE,
  DI_PLANE_RGB,

  DI_PLANE_Y,
  DI_PLANE_U,
  DI_PLANE_V,
  DI_PLANE_YUV
};

class DI_Image
{
public:
  DI_Image(DI_ImageFormat fmt = DI_IMAGE_UNKNOWN, int w = 0, int h = 0) 
    : m_format(fmt), m_width(w), m_height(h) {}

  class Plane {
  public:
    Plane() : m_data(0), m_linesize(0), m_type(DI_PLANE_UNKNOWN) {}

    void set(unsigned char * data, int linesize, DI_PlaneType type)
      { m_data = data; m_linesize = linesize; m_type = type; }
    
    unsigned char * line(int y) { return m_data + m_linesize * y; }
    const unsigned char * line(int y) const { return m_data + m_linesize * y; }

    unsigned char * m_data;
    int             m_linesize;
    DI_PlaneType    m_type;
  };

  void setFormatYUV420()
  {
    m_format = DI_IMAGE_YUV_420;
    m_planes[0].m_type = DI_PLANE_YUV;
  }

  void setFormatYUV420P()
  {
    m_format = DI_IMAGE_YUV_420P;
    m_planes[0].m_type = DI_PLANE_Y;
    m_planes[1].m_type = DI_PLANE_U;
    m_planes[2].m_type = DI_PLANE_V;
    m_planes[3].m_type = DI_PLANE_UNKNOWN;
  }

  /** Partially unimplemented function that returns the size of the
   * image data in bytes.*/

  unsigned size() const
  {
    unsigned s;

    unsigned pixels = m_width * m_height;

    if(m_format == DI_IMAGE_GRAYSCALE)
      s = pixels;
    else if(m_format == DI_IMAGE_RGB) 
      s = pixels * 3;
    else 
      s = 0;

    return s;
  }
  
  int width()  const { return m_width; }
  int height() const { return m_height; }

  Plane m_planes[4];
  DI_ImageFormat m_format;
  int m_width, m_height;
};


#endif
