/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_IO_MODE_H
#define DI_IO_MODE_H

/** Input/output modes. */
enum DI_IoMode
{
  /// All bits zero.
  DI_IO_NONE = 0x0,
  /// First bit set.
  DI_IO_INPUT = 0x1,
  /// Second bit set.
  DI_IO_OUTPUT = 0x2,
  /// Both first and second bit set.
  DI_IO_INPUT_OUTPUT = 0x3
};

/** Audio sample types. */
enum DI_AudioSampleFormat
{
  /// 16 bit two's complement audio samples.
  DI_ASF_INT16,
  /// 24 bit two's complement audio samples.
  DI_ASF_INT24,
  /** 32 bit two's complement audio samples.

      If this format is used for 24 bit samples, the samples must be
      MSB aligned.
  */
  DI_ASF_INT32,
  /// 32 bit floating point audio samples in the range [-1.0,1.0].
  DI_ASF_FLOAT32,
  /// 64 bit floating point audio samples in the range [-1.0,1.0].
  DI_ASF_FLOAT64
};

inline int DI_sampleWidth(DI_AudioSampleFormat format)
{
  switch(format) {
  case DI_ASF_INT16:
    return 2;
  case DI_ASF_INT24:
    return 3;
  case DI_ASF_INT32:
  case DI_ASF_FLOAT32:
    return 4;
  case DI_ASF_FLOAT64:
    return 8;
  }
  return 0;
}



#endif
