/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_VIDEO_1394_H
#define DI_VIDEO_1394_H

#include <di_video_input.h>

#include <libdc1394/dc1394_control.h>

#include <string>


class DI_Video1394 : public DI_VideoInput
{
public:
  
  class MultiCapture {
  public:

    void clear() { m_captures.clear(); }
    void addCapture(DI_Video1394 *);
    void capture(DI_Image **);
    uint size() const { return m_captures.size(); }

  protected:


    std::vector<dc1394camera_t *> m_captures;
    std::vector<DI_Video1394 *>   m_videos;
  };

  DI_Video1394();
  ~DI_Video1394();

  static void multiCapture(DI_Image **, DI_Video1394 **, uint n);

  virtual const DI_Image * captureImage();
  virtual void doneImage();

  /// The width of the video stream images.
  virtual int width() const;
  /// The height of the video stream images.
  virtual int height() const;
  /// The frame rate of the video stream.
  virtual float fps() const;
  /// Native image format of the stream.
  virtual DI_ImageFormat imageFormat() const;  
  virtual unsigned int size() const;

  virtual void setShutter(float value);
  virtual void setGain(float value);
  virtual void setExposure(float value);
  virtual void setBrightness(float value);
  virtual void setFocus(float value);
  virtual void setWhiteBalance(float u_to_blue, float v_to_red);
  void setFeature1394(int feature, float value);

  virtual void initialize(DI_ImageFormat fmt = DI_IMAGE_UNKNOWN, 
			  int width = DI_DONT_CARE, 
			  int height = DI_DONT_CARE, 
			  DI_FrameRate framerate = DI_FPS_IGNORE);

  virtual bool isInitialized() const;
  virtual void start();
  virtual bool isStarted() const;
  virtual void stop();
  virtual void finalize();

  void configure(const char * device, 
		 const char * camera = 0,
		 const char * euid = 0);

protected:
  bool enableCameraFeature(unsigned int feature, 
                           const std::string & description, 
                           bool automatic_mode,
                           unsigned int * feature_min_value,
                           unsigned int * feature_max_value);

   void switchToNearestFPS(quadlet_t compat, int current);

  std::string    m_videodevice;

  /** camera capture information. */
  dc1394camera_t * m_camera;

  /** camera information. */
  // dc1394_camerainfo m_camera_info;   
  /** camera feature information. */
  dc1394featureset_t m_features;
  /** camera misc info. */
  // dc1394_miscinfo m_camera_misc_info;

  u_int64_t m_euid;
  int m_cameraNum;

  int m_isoChannel;

  bool  m_initialized;
  bool  m_started;
  uint  m_fps;

  DI_Image m_image;
};

#endif
