/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_VIDEO_USB_PHILIPS_H
#define DI_VIDEO_USB_PHILIPS_H

#include <di_video_input.h>

#include <string>

#include <linux/videodev.h>

class DI_VideoUsbPhilips : public DI_VideoInput
{
public:
  DI_VideoUsbPhilips();
  ~DI_VideoUsbPhilips();

  virtual const DI_Image * captureImage();

  /// The width of the video stream images.
  virtual int width() const;
  /// The height of the video stream images.
  virtual int height() const;
  /// The frame rate of the video stream.
  virtual float fps() const;
  /// Native image format of the stream.
  virtual DI_ImageFormat imageFormat() const;  
  virtual unsigned int size() const;

  virtual void setShutter(float value);
  virtual void setGain(float value);

  virtual void initialize(DI_ImageFormat fmt = DI_IMAGE_UNKNOWN, 
			  int width = DI_DONT_CARE, 
			  int height = DI_DONT_CARE, 
			  DI_FrameRate framerate = DI_FPS_IGNORE);

  virtual bool isInitialized() const;
  virtual void start();
  virtual bool isStarted() const;
  virtual void stop();
  virtual void finalize();

  void configure(const char * device, const char * camera = 0);

  
protected:
  /**
    \brief Do ioctl(MCAPTURE), start grabbing frame in buffer
    \param frame The mmap buffer frame number (set to modulo total frames)
    \see #m_video_buffer.frames for frame count.

    This function will start the capture of an image into given mmap buffer
    with the current width & height
    
  */
  void mmapCapture(int frameNo);
  
  /** Wait for mmap capture to complete. "Releases" mmapCaptured frame. */
  void mmapSynchronize(int frameNo);

  bool m_initialized;

  int m_camerahandle;
  std::string m_devicename;
  std::string m_cameraname;

  video_mbuf m_video_buffers; 
  unsigned int m_videobuffersize; // local or mmapped size
  unsigned int m_videoformat;
  unsigned int m_currentFrame;
  unsigned int m_image_fps;
  
  unsigned char * m_videobuffer;
  std::vector<unsigned int> m_bufferoffsets;
  
  /** can we use select() in image capture */
  bool m_useSelect;
  /** can we use mmapped buffers in image capture? */
  bool m_useMMap;
  /** true if capturing is running */
  bool m_capturing;

  DI_Image         m_image;
};


#endif
