/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_APP_ACTIONS_H
#define MJ_APP_ACTIONS_H

#include <mj_app_action.h>

#include <qobject.h>
#include <qstring.h>

#include <string>

class MJ_ModuleDsp;

/** Action that starts the signal processing. */
class MJ_AppActionStartDsp : public MJ_AppAction
{
public:
  MJ_AppActionStartDsp(MJ_ModuleDsp *m) : m_module(m) {}
  virtual ~MJ_AppActionStartDsp();
  
  virtual void operate();
  virtual bool isIdentical(MJ_AppAction *);

protected:
  MJ_ModuleDsp *m_module;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Action that stops the signal processing. */

class MJ_AppActionStopDsp : public MJ_AppAction
{
public:
  MJ_AppActionStopDsp() {}
  virtual ~MJ_AppActionStopDsp();
  
  virtual void operate();
  virtual bool isIdentical(MJ_AppAction *);

protected:
  MJ_ModuleDsp *m_module;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/** Action that informs the UI thread that a module was modified. This
    class can be used to transmit both parameter and IO changes.  */

class MJ_AppActionModuleModified : public QObject, public MJ_AppAction
{
  Q_OBJECT
public:
  MJ_AppActionModuleModified(MJ_ModuleDsp *module, const char *parameter);
  MJ_AppActionModuleModified(MJ_ModuleDsp *module, uint ioChangeFlags);

  virtual ~MJ_AppActionModuleModified();
  
  virtual void operate();
  virtual bool isIdentical(MJ_AppAction *);

public slots:  
  void moduleDeleted();
protected:

  void connectSignals();

  MJ_ModuleDsp *m_module;
  std::string   m_parameter;
  uint          m_ioflags;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Action that shows an error message to the user. */

class MJ_AppActionErrorStr : public MJ_AppAction
{
public:
  MJ_AppActionErrorStr(const QString &s) : m_str(s.copy()) {}
  MJ_AppActionErrorStr(const char *s) : m_str(s) {}
  virtual ~MJ_AppActionErrorStr() {}
  
  virtual void operate();
  
protected:
  QString m_str;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Action that checks ifthe current DSP process has terminated. */

class MJ_AppActionCheckDsp : public MJ_AppAction
{
public:
  MJ_AppActionCheckDsp() {}
  virtual ~MJ_AppActionCheckDsp() {}
  virtual void operate();
};


#endif
