
/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_array3d.h>

#include <mj_error.h>
#include <mj_math.h>

template <class T>
MJ_Array3D<T>::~MJ_Array3D()
{ delete [] m_data; }

template <class T>
bool MJ_Array3D<T>::resize
(uint layers, uint rows, uint cols, bool fatalOnFailure, bool copy)
{
  uint elemsPerLayer = rows * cols;

  uint elems = elemsPerLayer * layers;

  if(elems == (m_layers * m_elemsPerLayer)) {

    m_layers = layers;
    m_rows   = rows;
    m_cols   = cols;
    m_elemsPerLayer = elemsPerLayer;

    return true;
  }

  T * data;

  if(elems) {
    
    data = new T[elems];
    
    if(!data) {
      if(fatalOnFailure)
	MJ_fatal(MJ_ERR_MEMORY, "MJ_Array3D<T>::resize");
      else
	return false;
    }

    if(copy) {
      for(uint i=0; i < MJ_min(m_layers, layers); i++)
	for(uint j=0; j < MJ_min(m_rows, rows); j++)
	  for(uint k=0; k < MJ_min(m_cols, cols); k++)
	    data[i * elemsPerLayer + j * cols + k] = get(i, j, k);
    }
  }
  else
    data = 0;
  
  delete [] m_data;

  m_layers = layers;
  m_rows   = rows;
  m_cols   = cols;
  m_elemsPerLayer = elemsPerLayer;
    
  m_data = data;

  return true;
}
