/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_array.h>
#include <mj_math.h>

extern const char *GC_ALG;
extern const char *GC_ALG_A;
extern const char *GC_ALGC;
extern const char *GC_ALR;

extern void MJ_arrayFatal(int);


/** Destructor. Deletes all array elements. */
template <class TElem>
MJ_Array<TElem>::~MJ_Array()
{ delete [] m_array; }

/// Get reference to an element.
template <class TElem>
TElem &MJ_Array<TElem>::get(uint nIndex)
{
  if(nIndex >= m_size) 
    MJ_arrayFatal(0);
  return m_array[nIndex];
}

/** Copy operator. Copies array elements one-by-one. */

template <class TElem>
MJ_Array<TElem> &MJ_Array<TElem>::operator = 
(const MJ_Array &xrArray)
{
  if(&xrArray == this) 
    return *this;

  resize(xrArray.m_size);
  
  for(uint nI=0; nI < m_size; nI++)
    m_array[nI] = xrArray.m_array[nI];
  return *this;
}

/** Resize the array. Old array elements are not lost. If the new
    array size matches the old size, then the array is not modified.

    @arg n The requested size of the array.

    @arg fatalOnFailure If this argument is true, then fatal error is
    generated if the memory allocation fails. 

    @return This function returns true if the memory allocation was
    successful and false if the memory allocation failed.
*/
template <class TElem>
bool MJ_Array<TElem>::resize(uint n, bool fatalOnFailure)
{
  if(n == m_size) return true;

  TElem *xpOld = m_array;

  if(n) {
    m_array = new TElem[n];
    if(!m_array) {
      if(fatalOnFailure) MJ_arrayFatal(2);
      else {
	m_array = xpOld;
	return false;
      }
    }

    uint nMax = MJ_min(n, m_size);
    for(uint nI=0; nI < nMax; nI++) m_array[nI] = xpOld[nI];
  }
  else
    m_array = 0;

  m_size = n;

  delete [] xpOld;

  return true;
}

