/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_BASE_H
#define MJ_BASE_H

#include <mj_error_enums.h>

class MJ_FailureDiagnose;

/** Base class for various Mustajuuri classes. This class is primarily
    used to get object-specific error and trace messages. The trace
    messages are supplemented with object type and pid. All calls are
    thread-safe.
    
    It contains little functionality besides the reporting. */
class MJ_Base
{
public:
  
  MJ_Base() {}
  virtual ~MJ_Base() {}


  /** @name Error reporting. */
  //@{
  void trace(const char *fname, const char *c2 = 0) const;
  void trace(const char *fname, int arg) const;
  void trace(const char *fname, void *v) const;
  void trace(const char *fname, int arg, int arg2) const;
  void trace2(const char *msg, ...) const;
  void trace3(bool show, const char *msg, ...) const;

  void error(MJ_ErrorCode errorCode, const char *s, const char *s2=0) const;
  void error(MJ_ErrorCode errorCode, const char *s, int arg) const;
  void error2(MJ_ErrorCode errorCode, const char *msg, ...) const;
  void error3(bool show, MJ_ErrorCode errorCode, const char *msg, ...) const;

  void failure(MJ_FailureDiagnose *) const;
  //@}

protected:
  virtual MJ_Base* mjParent();

};

#endif
