/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_CPU_COUNT_H
#define MJ_CPU_COUNT_H

#include <time.h>
#include <di_types.h>

/** CPU time counting. 

    Some application may need to know how much CPU time they have
    spent doing domething. The posix "clock" function tends to suck
    since it wraps around after some time.

    On systems where the time is measured with accuracy better than
    millisecond (very few) some error may accumulate over time.

    <B> Remember:</B> On most systems the real clock reso is around 10
    ms.  
*/
class MJ_CpuCount
{
public:
  ///
  MJ_CpuCount();

  ///
  virtual ~MJ_CpuCount();

  void reset();

  /// Update the CPU time count.
  void update();

  /** Return current CPU time usage in milliseconds.  
  */
  inline ulong cpuTimeMS() { return M_lSum; }

  /// Return the CPU time between two updates (millisec reso)
  inline ulong cpuTimeDiffMS() { return M_lDiff / 1000; }

  /// Return the CPU time between two updates (microsec reso)
  inline ulong cpuTimeDiffUS() { return M_lDiff; }

protected:

  clock_t  M_xLatest;
  ulong    M_lSum;
  ulong    M_lDiff;
};


#endif
