/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_DEBUG_H
#define MJ_DEBUG_H

#include <qstring.h>

/** Debugging targets. The list is currently highly incomplete. */
enum MJ_DebugTarget {
  /// Debug everything.
  MJ_DEBUG_ALL,
  MJ_DEBUG_DSO,
  MJ_DEBUG_KERNEL,
  MJ_DEBUG_KERNEL_GUI,
  MJ_DEBUG_MESSAGE,
  MJ_DEBUG_MESSAGE_ARRAY,
  MJ_DEBUG_MESSAGE_LIST,
  MJ_DEBUG_MODULE_DSP,
  MJ_DEBUG_MODULE_DSP_GUI,
  MJ_DEBUG_MODULE_DSP_GUI_POSTCARD,
  MJ_DEBUG_MODULE_DSP_UI,
  MJ_DEBUG_PLUGIN_BASE,
  MJ_DEBUG_PLUGIN_DSP,
  MJ_DEBUG_PLUGIN_LOADER,
  MJ_DEBUG_RT_DRIVER,
  MJ_DEBUG_RT_THREAD,
  /// Allow test menus to exist in various windows.
  MJ_DEBUG_TEST_MENUS,
  /// Query debugging targets.
  MJ_DEBUG_WHAT,
  /// Quert debugging targets (another option)
  MJ_DEBUG_QMARK,
  MJ_DEBUG_SIZEOF
};

///
extern const char *MJ_debugTargetKeys[MJ_DEBUG_SIZEOF];

/** Handles debugging. 
    
    This class knows how to switch on (part) of the debugging
    variables (for example MJ_ModuleDsp::m_moduleDspDebug).

    It also contains a list of keywords that represent some debugging
    target. 

    An example: User can give additional arguments via the command
    line that specify which plugins should be debugged. The plugin
    identifier string is used as the key with which we identify
    plugins. As a plugin is loaded its debugging variable (if
    specified by the plugin) is modified to "true".
    
    This class is probably only useful with Mustajuuri.  

    For examples of use see mj/main.C and mj_plugin_loader_tmpl.C.  */
class MJ_Debug
{
public:
  /** Add debugging for some thing that relates to a given
      string. There are some default strings that match most common
      debugging targets. If string is not found in the list of default
      targets it is appended to the list of debugging strings. */
  static bool debugName(const char *);

  /** Debug some default type. */
  static bool debugType(MJ_DebugTarget);

  /** Returns true if the string can be found in the list of debugging
      targets. */
  static bool debugNeeded(const char *);

  /** Returns true if global debugging is switched on. */
  static int debugAll() { return m_debugAllv; }

  /** Controls default debugging mode. */
  static void setDebugAll(int debugAll); 
  
  /** Allows test menus. */
  static bool allowTestMenus() { return m_allowTestMenus; }

  /** Write the known debug strings to screen. */
  static void writeDebugStrs();
protected:
  ///
  static int m_debugAllv;
  static bool m_allowTestMenus;
};


#endif
