/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_DSP_OPTIONS_H
#define MJ_DSP_OPTIONS_H

enum MJ_DspOption {
  /// All sample rates must be equal.
  MJ_DSP_EQUAL_SAMPLE_RATE_IO,

  /// Maximum input sampling rate
  MJ_DSP_MAX_SAMPLE_RATE_IN,
  MJ_DSP_MAX_SAMPLE_RATE_OUT,
  MJ_DSP_MIN_SAMPLE_RATE_IN,
  MJ_DSP_MIN_SAMPLE_RATE_OUT,

  /// The number of input channels must equal the number of output channels.
  MJ_DSP_EQUAL_CHANNEL_IO,
  /// Maximum number of input channels
  MJ_DSP_MAX_CHANNELS_IN,
  MJ_DSP_MAX_CHANNELS_OUT,
  MJ_DSP_MIN_CHANNELS_IN,
  MJ_DSP_MIN_CHANNELS_OUT,
  MJ_DSP_SIZEOF
};

extern const char *MJ_DspOptionWords[MJ_DSP_SIZEOF];

extern const char *MJ_DspOptionWords2[MJ_DSP_SIZEOF];

/** DSP plugin options. 
    
    This class contains DSP-dpecific options (allowed samplign rates)
    for plugins. The values are stored in an array of integers. An
    array on bools tells which parameters are truly defined. After
    construction, each parameter has value zero and is undefined. 

*/

class MJ_DspOptions
{
public:
  ///
  MJ_DspOptions();
  ///
  virtual ~MJ_DspOptions();
  
  bool setValue(MJ_DspOption, int);

  void undefine(MJ_DspOption);

  int value(MJ_DspOption) const;

  bool isDefined(MJ_DspOption) const;
  
  void defaultChans(int chans);

  void defaultChans2(int chansIn, int chansOut);

  void setChans(int minIn, int maxIn, int minOut, int maxOut);
  
  void setInChans(int minIn, int maxIn);

  void setOutChans(int minIn, int maxIn);

  void undefineAll();

  /** Check that a given option index is ok. */
  bool checkArg(MJ_DspOption) const;

  /** Determines that this IO configuration is ok. */
  bool acceptIoChans(int in, int out);

  /** */
  bool acceptInputChans(int in);

  /** Given a proposed number of input channels, determines the
      closest match to that number. */
  int newInChans(int propose);
  /** Given a proposed number of output channels, determines the
      closest match to that number. */
  int newOutChans(int propose);
  
  /** Check that given sampling rates are ok. */
  bool acceptIoSr(int in2, int out);

  /// Returns true if the given arg exceeds the parameter value
  bool exceeds(MJ_DspOption, int) const;
  /// Returns true if the given arg is below the parameter value
  bool below(MJ_DspOption, int);

protected:

  int    m_dataArray[MJ_DSP_SIZEOF];
  bool   m_defined[MJ_DSP_SIZEOF];
};

#endif
