/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_DSP_PLUGIN_INFO_GUI_H
#define MJ_DSP_PLUGIN_INFO_GUI_H

#include <mj_dsp_options.h>

#include <qwidget.h>

class MJ_PluginDbeDsp;
class MJ_PluginDbeDspLoader;

class QCheckBox;
class QComboBox;
class QGridLayout;
class QLabel;

/** A GUI for viewing plugin information. */
class MJ_DspPLuginInfoGui : public QWidget
{
  Q_OBJECT
public:
  MJ_DspPLuginInfoGui(MJ_PluginDbeDsp *dbe = 0,
		      QWidget      *parent = 0,
		      const char   *name = 0);
  virtual ~MJ_DspPLuginInfoGui();

  void setDbe(MJ_PluginDbeDsp *dbe);

public slots:
  void setDebug(bool);
protected:

  void drawChildWidgets();
  
  void addLabels(QGridLayout *grid, 
		 QLabel **labels, 
		 int row, 
		 const QString & name);

  MJ_PluginDbeDsp *m_dbe;

  QLabel *m_authors[2];
  QLabel *m_copyright[2];
  QLabel *m_about[2];
  QLabel *m_functionality[2];
  QLabel *m_metafunctionality[2];
  QLabel *m_menuPos[2];
  QLabel *m_urlHome[2];
  QLabel *m_urlHelp[2];
  QLabel *m_library[2];
  QLabel *m_compileTime[2];

  QLabel *m_optParams[MJ_DSP_SIZEOF];

  QCheckBox *m_debug;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** A GUI for viewing information on many plugin types. */
class MJ_PluginsInfo : public QWidget
{
  Q_OBJECT
public:
  MJ_PluginsInfo(MJ_PluginDbeDspLoader *loader,
		 QWidget      *parent = 0,
		 const char   *name = 0);

  virtual ~MJ_PluginsInfo() {}


  // void hide();

public slots:
  void selectDbe(int);

protected:

  MJ_PluginDbeDspLoader *m_loader;
  QComboBox             *m_combo;
  MJ_DspPLuginInfoGui   *m_pluginGui;
};


#endif
