/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_FAILURE_DIAGNOSE_H
#define MJ_FAILURE_DIAGNOSE_H

/** Class to hold error diagnostics. */

#include <mj_error.h>

#include <qstring.h>
#include <qptrlist.h>

#include <stdio.h>

class MJ_Base;

class QTextStream;

class MJ_FailureDiagnose 
{
public:
  MJ_FailureDiagnose();
  virtual ~MJ_FailureDiagnose();

  void failure(const MJ_Base *origin = 0,
	       MJ_ErrorCode  code = MJ_ERR_UNKNOWN,
	       MJ_ErrorLevel level = MJ_ERRL_ERROR);


  void addMessage(const QString &);
  void addMessage(const char *);
  // void addMessage(ostrstream *);

  void clear();

  /// The object that detected the error.
  inline const MJ_Base *origin() { return m_origin; }
  void setOrigin(MJ_Base *o) { m_origin = o; }

  inline MJ_ErrorCode  code() const { return m_code; }
  void setCode(MJ_ErrorCode);

  inline MJ_ErrorLevel level() const { return m_level; }
  void setLevel(MJ_ErrorLevel);

  QList<QString> & messages() { return  m_messages; }

  bool isEmpty() const;

  void write(QTextStream *);

  void toUser();

  static void setTraceFile(FILE *);
  
protected:

  const MJ_Base*m_origin;
  MJ_ErrorCode  m_code;
  MJ_ErrorLevel m_level;
  int           m_pid;

  QList<QString>   m_messages;

};



#endif
