/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MALLOC_H
#define MJ_MALLOC_H

#include <mj_allocator.h>

#include <qptrlist.h>

#include <di_types.h>



/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Memory allocator. */
class MJ_Malloc : public MJ_Allocator
{
public:

  class Unit
  {
  public:
    Unit(ulong bytes);
    ~Unit();

    /// Number of bytes available
    inline ulong available() const { return m_bytes - m_used; }
    /// Number of bytes allocated
    inline ulong totalBytes() const { return m_bytes; }
    
    void *allocate(ulong bytes);
    
    /** Default allocation block size. The value returned should be
	large enough to (over)fill any cache. */
    static ulong defaultSize() { return 1 << 21; }
    
  protected:
    void *m_data;
    ulong m_bytes;
    ulong m_used;
    
  private:
    /// Disabled
    Unit(const Unit &) {}
    Unit &operator = (const Unit &) { return *this; }
  };

  MJ_Malloc() { m_units.setAutoDelete(true); }
  virtual ~MJ_Malloc() {}

  /// Free all allocated memory.
  void free() { m_units.clear(); }

  void *allocate(ulong bytes);

  /// Number of allocation units used.
  uint units() const { return m_units.count(); }

protected:
  QList<Unit> m_units;
private:
  /// Disabled
  MJ_Malloc(const MJ_Malloc &) {}
  MJ_Malloc &operator = (const MJ_Malloc &) { return *this; }
};

#endif
