/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MESSAGE_PASS_H
#define MJ_MESSAGE_PASS_H

#include <mj_misc.h>

#include <qptrlist.h>

class MJ_Message;
class MJ_ModuleDsp;
class QTextStream;

/** Remapping information for a message. 

    As a message is passed from one module to another it is necessary
    to readdress the message. This readdressing information is stored
    in MJ_MessageRemap objects. MJ_MessageRemap objects are also used
    to do the transmission.

 */
class MJ_MessageRemap
{
public:
  MJ_MessageRemap();

  ~MJ_MessageRemap();

  void set(const char * from, 
	   const char * to, 
	   const char * sourceId, 
	   const char * targetId);
  
  /// Returns "from" address (message id).
  const char * from() const { return m_from; }
  /// Sets the "from" address (message id)
  void setFrom(const char * f) 
  { MJ_delAndDup(&m_from, f); }
  
  /// Returns "to" address (message id).
  const char * to() const { return m_to; }
  /// Sets the "to" address (message id)  
  void setTo(const char * f) 
  { MJ_delAndDup(&m_to, f); }

  /// Returns target module.
  MJ_ModuleDsp *target() { return m_target; }
  /// Sets target module
  void setTarget(MJ_ModuleDsp *t);

  /// Returns source module id.
  const char * sourceId() { return m_sourceId; }
  /// Returns target module id.
  const char * targetId() { return m_targetId; }

  bool deliver(const char *, MJ_Message *);
  bool equals(const MJ_MessageRemap &);

  bool write(QTextStream *s);

  bool read(QTextStream *s);

  /** Key string. */
  static const char *key() { return m_key; }

protected:

  char *        m_from,  * m_to;
  MJ_ModuleDsp *m_target;
  char *        m_sourceId;
  char *        m_targetId;

  static const char * m_key;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** List of MJ_MessageRemap. Introduces a few new methods to QList. */

class MJ_MessageRemapList : public QList<MJ_MessageRemap>
{
public:
  /** Returns the index of the remapper with the given target. */
  int findTarget(MJ_ModuleDsp *);
  // int findTarget(int id);
  int findRemap(MJ_MessageRemap *);
};

#endif
