/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MODULE_DSP_GUI_H
#define MJ_MODULE_DSP_GUI_H

#include <mj_gui_enums.h>

#include <mj_module_dsp_ui.h>

#include <qapplication.h>
#include <qframe.h>

class MJ_MessageCarrier;
class MJ_ModuleDsp;
class MJ_PluginDbeDsp;

/** Base class for Mustajuuri DSP module GUIs. */
class MJ_ModuleDspGui : public QFrame, public MJ_ModuleDspUi
{
  Q_OBJECT

public:

  MJ_ModuleDspGui(MJ_RtControl *rtparent,
		  MJ_ModuleDsp *module,
		  QWidget      *parent = 0,
		  const char   *name = 0);
  
  virtual ~MJ_ModuleDspGui();

  virtual void hide();

  virtual int weight();

  void setDbe(MJ_PluginDbeDsp *dbe);

  ///
  static void setDspGuiDebug(bool b) { m_dspGuiDebug = b; }

public slots:
 
  virtual void moduleModified(const char *);
  virtual void moduleIoModified(uint);

  void moduleDestroyed();

  void showModule() { emit emitShowModule(m_module); }

signals:
  void emitShowModule(MJ_ModuleDsp *);
  /** Emitted as the widget is destroyed. */
  void emitDestroyed(QWidget *);
protected:

  void defaultConnections();

  virtual MJ_Base* mjParent();

  static bool m_dspGuiDebug;
};

#endif
