/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MODULE_DSP_UI_H
#define MJ_MODULE_DSP_UI_H

#include <mj_base.h>
#include <mj_misc.h>
#include <mj_rt_control.h>

class MJ_ModuleDsp;
class MJ_PluginDbeDsp;
class MJ_MessageCarrier;

/** Base class for module user interfaces. */
class MJ_ModuleDspUi : public MJ_RtControl, public MJ_Base
{
public:
  MJ_ModuleDspUi(MJ_RtControl *rtparent,
		 MJ_ModuleDsp *module);
  virtual ~MJ_ModuleDspUi();

  MJ_ModuleDsp *module() { return m_module; }

  /** Sends a message to the object. Messages are not deleted. */
  bool deliverMessage(MJ_MessageCarrier *msg, MJ_MessageCarrier *counterMsg = 0);

  bool msgDeliverString(const char *, const QString &);

  /** Delivers a double to the module. If possible, the system tries
      to validate the value, so it fits to the allowed range. */
  bool msgDeliverDouble(const char *, double);

  /// Deliver an empty message
  bool msgDeliverEmpty(const char *);

  /// Add undo information 
  void addUndoInfo(MJ_MessageCarrier *msg, 
		   MJ_MessageCarrier *counterMsg,
		   const QString &name = "",
		   MJ_Unit unit = MJ_UNIT_BLANK);

  /// Save the module state to a file.
  virtual bool saveModule(const QString &);

  /** Load the module state from a file. The IO definitions of the
      module are not changed. */
  virtual bool loadModule(const QString &);

  bool startThreaded(long firstFrame = -1, long lastFrame = -1);
  bool stopPlaying();

  static void setUiDebug(bool b) { m_dspUiDebug = b; }

  static MJ_ModuleDspUi * findUi(const MJ_ModuleDsp *);

protected:
  /** The Data Base Entry that created this module. */
  MJ_PluginDbeDsp          *m_dbe;

  MJ_ModuleDsp *m_module;

  static bool m_dspUiDebug;

};

#endif
