/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_parameter_numeric.h>

#include <mj_mapper.h>
#include <mj_message_numeric.h>
#include <mj_message_carrier.h>
#include <mj_tr.h>

#include <qtextstream.h>

template <class TElem>
MJ_ParameterNumeric<TElem>::MJ_ParameterNumeric()
  : m_min(0),
    m_max(0),
    m_value(0),
    m_mapper(0)
{
  m_allowedValues.setAutoDelete(true);
}

template <class TElem>
MJ_ParameterNumeric<TElem>::~MJ_ParameterNumeric()
{
  setMapper(0);
}

template <class TElem>
QStringList MJ_ParameterNumeric<TElem>::writeClean()
{
  QStringList l = MJ_Parameter::writeClean();

  l.append(MJ_appendUnit(name(), m_unit));
  l.append(QString().setNum(m_value));
  QString buf;
  QTextOStream(&buf) << "[" << m_min << " , " << m_max << "]" ;

  l.append(buf);
 
  return l;
}

template <class TElem>
MJ_MessageCarrier *MJ_ParameterNumeric<TElem>::valueMessage()
{
  MJ_MessageNumeric<TElem> mn(m_value);
  MJ_MessageCarrier *mi = new MJ_MessageCarrier(m_id, & mn);
  return mi;
}

template <class TElem>
bool MJ_ParameterNumeric<TElem>::setValue(TElem value)
{
  if(value > m_max) value = m_max;
  else if(value < m_min) value = m_min;
    
  m_value = value;
  return true;
}

template <class TElem>
void MJ_ParameterNumeric<TElem>::setRange(TElem xMin, TElem xMax)
{
  m_min = xMin;
  m_max = xMax;
}

/** Sets the mapper to be used in the user interface. */
template <class TElem>
void MJ_ParameterNumeric<TElem>::setMapper(MJ_Mapper *m)
{
  if(m_mapper) delete m_mapper;
  m_mapper = m;
}
