/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_PARAMETERS_H
#define MJ_PARAMETERS_H

#include <mj_parameter_numeric.h>

#include <mj_midi.h>

/** MIDI parameter. */
class MJ_ParameterMidi : public MJ_Parameter
{
public:

  MJ_ParameterMidi() {}

  MJ_ParameterMidi(const char *name, const char *id) 
    : MJ_Parameter(name, id) {}

  virtual ~MJ_ParameterMidi();
  
  MJ_MessageType type() const;
  MJ_MessageCarrier *valueMessage();

  MJ_MidiEvent & event() { return m_event; }

protected:
  MJ_MidiEvent m_event;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class QStringList;

/** String (unicode) message. 

    The parameter may also contain information regarding which strings
    are allowed values. This information is accessed via calls to
    allowedStrsand modofied via calls to addAllowedStr. */
class MJ_ParameterString : public MJ_Parameter
{
public:
  MJ_ParameterString();
  MJ_ParameterString(const QString &str);
  MJ_ParameterString(const QString &str, 
		     const QString &name,
		     const char    *id);
  MJ_ParameterString(const QString &str, 
		     const char    *name,
		     const char    *id);

  virtual ~MJ_ParameterString();

  virtual MJ_MessageType type() const;

  MJ_MessageCarrier *valueMessage();

  void setString(const QString &s) { m_string = s; }

  const QString & string() const { return m_string; }

  virtual QStringList writeClean();

  /** List of allowed values or NULL if there is no list. */

  QStringList *allowedStrs() { return m_allowedStrs; }

  void addAllowedStr(const QString &);
  void addAllowedStr(const char *);
protected:
  QString m_string;
  QStringList *m_allowedStrs;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Filename message. This is a string parameter, but it is extended
    with the "filter" argument that is used to select files. */
class MJ_ParameterFilename : public MJ_ParameterString
{
public:

  MJ_ParameterFilename(const QString & string,
		       const QString & name,
		       const char * id,
		       const QString &filter, 
		       int mode);

  MJ_ParameterFilename(const QString & string,
		       const char * name,
		       const char * id,
		       const QString &filter, 
		       int mode);

  virtual ~MJ_ParameterFilename();

  /// File open mode for Qt
  int mode() const { return m_mode; }

  /// File name filter
  const QString & filter () const { return m_filter; }

protected:
  QString m_filter;
  int     m_mode;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Bool parameter. 

@see MJ_MessageBool
*/
class MJ_ParameterBool : public MJ_Parameter
{
public:

  MJ_ParameterBool(bool value,
		   const QString & name,
		   const char * id);

  MJ_ParameterBool(bool value,
		   const char * name,
		   const char * id);

  virtual ~MJ_ParameterBool();

  virtual MJ_MessageType type() const;

  MJ_MessageCarrier *valueMessage();

  inline bool value() const { return m_value; }
  inline void setValue(bool value) { m_value = value; }
  
protected:
  bool m_value;
};

#endif
