/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_PATHS_H
#define MJ_PATHS_H

#include <qstring.h>

enum MJ_Path {
  MJ_PATH_MUSTAJUURI,
  MJ_PATH_MUSTAJUURI_PLUGIN,
  MJ_PATH_PRIOGIVE,
  MJ_PATH_RTC_HZ,
  MJ_PATH_SIZEOF
};

extern const char * MJ_PathKeys[MJ_PATH_SIZEOF];

/** UNIX path handling. 

    Path names have the form: 
    /dir1/dir2/dir3/:/anotherdir1/anotherdir2/
 */
class MJ_Paths
{
public:
  MJ_Paths();
  ~MJ_Paths();

  /// Checks if an environment variable exists.
  static bool isEnvVariable(const QString &);

  bool addPathVariable(const QString &);
  bool addPathVariableN(MJ_Path);

  /// Append a path to current path list;
  void addPath(const QString &);
  
  void clear() { m_path.truncate(0); }

  /// Return nth path name from the path description.
  QString path(uint);
	
  /// Nunmber of individual paths.
  uint paths();

  /// Find the first occurence of a file name in the path.
  QString findFile(const QString &);

  /// Paths string.
  QString & str() { return m_path; }

  static QString homeDir();

protected:
  QString m_path;
};

#endif
