/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_PLUGIN_MENU_H
#define MJ_PLUGIN_MENU_H

#include <qwidget.h>
#include <qmenubar.h>
#include <qlabel.h>

#include <mj_plugin_db_tree.h>

class MJ_PluginMenu;
class MJ_PluginDbeDspLoader;
class MJ_PluginFilter;

/** Used for constructing a menu of modules. 

    Example usage: 

\begin{verbatim}
  QPopupMenu *setups = new QPopupMenu;
  MJ_PluginDbTree_GQ moduleMenuBuilder;
  moduleMenuBuilder.setNode(m_kernel->dsPluginTree());
  moduleMenuBuilder.formTree(this, SLOT(newModule(const QString &)), setups);
\end{verbatim}

 */
class MJ_PluginDbTree_GQ : public QObject
{
public:
  MJ_PluginDbTree_GQ() : m_node(0), m_menu(0) {}
  ~MJ_PluginDbTree_GQ() {}

  void setNode(MJ_PluginDbTreeNode *node)
  { m_node = node; }

  bool formTree(QObject       *callObj,
		const char    *callSlot,
		QPopupMenu    *menu,
		MJ_PluginFilter *filter = 0);

protected:
  static uint filteredNodesLeft(MJ_PluginDbTreeNodeMany *node, 
				uint start,
				MJ_PluginFilter *f)
  { 
    uint r = 0; 
    for(uint i=start; i < node->nofNodes(); i++) 
      if(node->getNode(i)->filter(f)) r++;
    return r;
  }

  MJ_PluginDbTreeNode  *m_node;
  QPopupMenu           *m_menu;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// A test class for showing all plugins in a menu.
class MJ_PluginMenu : public QWidget
{
  Q_OBJECT
public:
  MJ_PluginMenu(QWidget *parent=0, 
		const char *name=0,
		MJ_PluginDbeDspLoader *xpLoader = 0);
  ~MJ_PluginMenu();
  
public slots:
  void call(const QString &);

signals:
  void emitName(const QString &);
protected:

  QMenuBar                 *menu;
  MJ_PluginDbeList         *M_xpDB;
  MJ_PluginDbeDspLoader    *M_xpLoader;

  QList<QObject>            M_xObjs;
};

#endif
