/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_RT_DRIVER_H
#define MJ_RT_DRIVER_H

#include <mj_malloc.h>
#include <mj_module_dsp.h>
#include <mj_module_rtaudio.h>
#include <mj_error.h>

/** Drives some module in real-time. 

    This class IS NOT responsible for master thread/mutex stuff.
*/
class MJ_RtDriver : public MJ_Base
{
public:
  MJ_RtDriver(MJ_ModuleDsp *m = 0);
  virtual ~MJ_RtDriver();

  /** Sets the module in use. */
  void setModule(MJ_ModuleDsp *m);

  MJ_ModuleDsp *module() { return m_mainModule; }

  /** Runs the module for n frames. */
  bool runPeriods(ulong periods);

  /// Request stop - only for applies to "runFrames".
  void requestStop() { m_stop = true; }

  /** Add an error message. This class collects error messages. They
      can be later sent to the user etc. */
  void addError(MJ_FailureDiagnose *);
  
  float fillOutputs(float fillMs);
  float fillInputs(float fillMs);

  /// Debug this class
  static void setDebug(bool b) { m_debug = b; }

  bool prepare(long firstFrame = 0, long lastFrame = 0);
  bool prepareFast();
  void processPeriod();
  bool stop();

  void setMinBuffer(float ms) { m_minBufferMS=ms;}

  double lowestBuf(bool out = true);
  uint nofAudioIos() const { return m_audioIos.size(); }

  void *allocate(ulong bytes)
  { return m_malloc.allocate(bytes); }

  MJ_Malloc *dspMalloc() { return & m_malloc; }

  long firstFrame() const { return (long) m_firstFrame; }

protected:

  bool prepareInternal();

  void processRt(ulong frames);

  void getChildren();
  bool checkConflicts();

  MJ_ModuleDsp *m_mainModule;
  bool          m_stop;
  ulong         m_firstFrame;
  ulong         m_lastFrame;
  
  /// Children of the main module.
  QList<MJ_ModuleDsp> m_children;

  MJ_Array<MJ_ModuleRtAudio *> m_audioIos;
  QList<MJ_FailureDiagnose> m_errors;

  MJ_Malloc     m_malloc;

  float         m_minBufferMS;

  static bool m_debug;
};

#endif
