/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_RUN_STATUS_H
#define MJ_RUN_STATUS_H

#include <qtimer.h> 
#include <qwidget.h>

class MJ_ModuleDsp;
class MJ_ModuleDspUi;
class MJ_RtDriver;
class MJ_RtThread;
class MJ_LevelDisplay;
class MJ_GraphicsArea;
class MJ_GraphicPolygon;
class MJ_GraphicText;

class QLabel;
class QApplication;
class QPushButton;
class QButton;

/** A clock widget. Uses multiple labels to show the time. */

class MJ_LcdClock : public QWidget
{
  Q_OBJECT
public:  

  MJ_LcdClock(QWidget      *parent = 0,
	      const char   *name = 0);

  virtual ~MJ_LcdClock();

  enum unit {
    MJ_LCD_HOURS,
    MJ_LCD_MINUTES,
    MJ_LCD_SECONDS,
    MJ_LCD_FRAGMENTS,
    MJ_LCD_SIZEOF    
  };

  void clear();

  void set(int hour, int min, int sec, int frag);
  void setNumber(int,int);

  void setMaxFrag(int);

  static void test(QApplication *);

protected:
  int m_fragMax, m_fragDigits;

  int m_time[MJ_LCD_SIZEOF];
  
  /// 
  QLabel *m_timeDisplays[MJ_LCD_SIZEOF];
  
  /// Allocate a string here to avoid constant de/allocation.
  QString m_tmp;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** A widget that shows the running status of Mustajuuri. */

class MJ_RunStatus : public QWidget
{
  Q_OBJECT
public:  

  MJ_RunStatus(QWidget      *parent = 0,
	       const char   *name = 0);

  virtual ~MJ_RunStatus();
  
  bool startThreaded(MJ_ModuleDspUi *);

  void started(MJ_ModuleDspUi *, MJ_RtThread *);
  void stopped();
  void moduleDestroyed(MJ_ModuleDsp *);

public slots:
  void stop();
  void panic();
  void restart();
  void update();


  void grMousePress(QMouseEvent *, float, float);

protected:
  void updateBuffer();

  QTimer            m_timer;

  MJ_LcdClock      *m_time;
  QPushButton      *m_panic;
  QButton          *m_stop;
  QButton          *m_restart;
  MJ_LevelDisplay  *m_load;
  QLabel           *m_loadText;
  MJ_GraphicsArea  *m_bufArea;
  MJ_GraphicPolygon *m_bufCurve;
  MJ_GraphicPolygon *m_bufRange;
  MJ_GraphicText    *m_underflowWarn;

  MJ_ModuleDsp     *m_mainModule;
  MJ_ModuleDspUi   *m_mainUi;

  uint             m_updates;

  MJ_RtThread      *m_thread;
};

#endif
