/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_SO_REF_COUNT_H
#define MJ_SO_REF_COUNT_H

#include <qstring.h> 

/** Shared object reference counter. 

    This class is used for closing a particular shared lib that was
    opened with a dlopen-call. */
class MJ_SoRefCount
{
public:
  // MJ_SoRefCount() : m_plugins(0), M_Handle(0) { m_objects++; }
  MJ_SoRefCount(void *handle, const QString &file);

  ~MJ_SoRefCount();
  
  void incrementReferenceCounter() { m_plugins++; }
  void decrementReferenceCounter() { m_plugins--; }

  void setHandle(void *handle) { m_handle = handle; }

  void setFileName(const QString &file) { m_fileName = file; }
  const QString & fileName() const { return m_fileName; }
  
  int count() const { return m_plugins; }
  
  /// Number of MJ_SoRefCount objects floating around.
  static int soCount() { return m_objects; }
  
  static void setDebug(bool b) { m_debug = b; }

protected:
  int               m_plugins;
  
  QString           m_fileName;

  /* Handle to the shared library that was used to get this module.*/
  void             *m_handle;

  static int        m_objects;

  static bool       m_debug;
};

#endif
