/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_TIME_STAMP_H
#define MJ_TIME_STAMP_H

#include <di_time_stamp.h>

/** Time stamp. Extends DI_TimeStamp by 

    <OL> 

    <LI> adding a boolean that can hold real-time data

    <LI> by adding frame size information. 
    
    </OL>
*/
class MJ_TimeStamp : public DI_TimeStamp
{
public:
  MJ_TimeStamp(uint  frameSize = 128) : m_frameSize(frameSize), 
    m_isRealTime(false) {}
  ~MJ_TimeStamp() {}

  inline void setRealTime(bool b) { m_isRealTime = b; }
  inline bool isRealTime() const { return m_isRealTime; }

  inline void setFrameSize(uint fs) { m_frameSize = fs; }
  inline uint frameSize() const { return m_frameSize; }
  /// Duration of one frame in seconds
  inline double frameDurationS() const
    { return (double) m_frameSize / (double) m_hz; }

  inline bool operator < (const MJ_TimeStamp & that) const
  { return m_stamp < that.m_stamp; }

protected:
  uint  m_frameSize;
  bool  m_isRealTime;

};

#endif
