/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_TIME_UTILS_H
#define MJ_TIME_UTILS_H

#ifndef WIN32
#include <unistd.h>
#include <sys/time.h>
#else
#include <windows.h>
#include <mmsystem.h>
//#include <winsock2.h>
#endif
#include <di_types.h>

#define MJ_BILLION 1000000000
#define MJ_MILLION 1000000


inline void MJ_addTimeNs(struct timespec *tspec, long ns)
{
  tspec->tv_nsec += ns;
  if(tspec->tv_nsec >= MJ_BILLION) {
    long tmp = tspec->tv_nsec / MJ_BILLION;
    tspec->tv_sec += tmp;
    tspec->tv_nsec = tspec->tv_nsec - tmp * MJ_BILLION;
  }
}

inline void MJ_addTimeUs(struct timeval *tspec, long us)
{
  tspec->tv_usec += us;
  if(tspec->tv_usec >= MJ_MILLION) {
    long tmp = tspec->tv_usec / MJ_MILLION;
    tspec->tv_sec += tmp;
    tspec->tv_usec = tspec->tv_usec - tmp * MJ_MILLION;
  }
}

inline void MJ_addTime(struct timeval *tspec, const struct timeval *tspecAdd)
{
  tspec->tv_sec  += tspecAdd->tv_sec;
  tspec->tv_usec += tspecAdd->tv_usec;
  if(tspec->tv_usec >= MJ_MILLION) {
    long tmp = tspec->tv_usec / MJ_MILLION;
    tspec->tv_sec += tmp;
    tspec->tv_usec = tspec->tv_usec - tmp * MJ_MILLION;
  }
}

inline long MJ_timeDiffNs(const struct timespec *tspecOld,
			  const struct timespec *tspecNew)
{
  return (tspecNew->tv_sec - tspecOld->tv_sec) * MJ_BILLION +
    tspecNew->tv_nsec - tspecOld->tv_nsec;
}

inline long MJ_timeDiffUs(const struct timeval *tspecOld,
			  const struct timeval *tspecNew)
{
  return (tspecNew->tv_sec - tspecOld->tv_sec) * MJ_MILLION +
    tspecNew->tv_usec - tspecOld->tv_usec;
}

inline long MJ_timeDiff(const struct timeval *tspecOld,
			const struct timeval *tspecNew,
			long resoHz)
{
  return (tspecNew->tv_sec - tspecOld->tv_sec) * resoHz +
    (tspecNew->tv_usec - tspecOld->tv_usec) / (MJ_BILLION / resoHz);
}

#endif
