/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_UNDO_CENTER_H
#define MJ_UNDO_CENTER_H

#include <mj_base.h>
#include <mj_int_remap.h>
#include <mj_list.h>
#include <mj_undo_info.h>

#include <qobject.h>

class MJ_Satellite;

class QPopupMenu;

/** A utility class for undo/redo operations. 
    
    The correct usage of this class is to give it a menu where the
    actions will be presented. After that you must add the satellites
    of the DspModules to be managed. Perform these two operations in
    this order.  

    @see MJ_UndoInfo
*/

class MJ_UndoCenter : public QObject, public MJ_Base
{
  Q_OBJECT

public:
  /// Default constructor.
  MJ_UndoCenter();
  /// Desctructor
  virtual ~MJ_UndoCenter();

  /** Sets the menu in which the undo-options will be added.

      As this method is called four new elements are inserted to the menu. 
      "undo", "undo latest", "redo" and "redo latest".
   */
  void setMenu(QPopupMenu *editMenu);
  /** Adds a satellite to listen to. The module UIs pass the undo
      information to the satellite. The satellite in turn emits the
      undo packages. Thsi method creates the necessary link. */
  void addSatellite(MJ_Satellite *s);

public slots:

  /// Add one undo information item.
  void addUndo(MJ_UndoInfo *);
  /// Remove one undo information item.
  void removeUndo(MJ_UndoInfo *);

  /// Add one redo information item.
  void addRedo(MJ_UndoInfo *);
  /// Remove one redo information item.
  void removeRedo(MJ_UndoInfo *);

  /// Undo nth operation in the nudo stack.
  void undo(int);
  /// Redo nth operation in the nudo stack.
  void redo(int);

  /// Undo the latest operation
  void undoLatest();
  /// Redo the latest operation
  void redoLatest();

protected:

  /// For undo, redo ops.
  void addAction(MJ_UndoList &doList, 
		 MJ_List<MJ_IntRemap> &remaps,
		 QPopupMenu *menu,
		 const char *slotName,
		 MJ_UndoInfo *i);

  /// For undo, redo ops.
  void rebuildMenu(MJ_UndoList &doList, 
		   MJ_List<MJ_IntRemap> &remaps,
		   QPopupMenu *menu,
		   const char *slotName);

  QPopupMenu      *m_undoMenu, *m_redoMenu, *m_edit;
  int              m_undoId, m_undoLatestId, m_redoId, m_redoLatestId;

  MJ_UndoList        m_undos;
  MJ_UndoList        m_redos;
  MJ_List<MJ_IntRemap> m_undoRemaps;
  MJ_List<MJ_IntRemap> m_redoRemaps;

  static int         m_debug;
};

#endif
