/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_MESSAGE_NUMERIC_H
#define MJ_MESSAGE_NUMERIC_H

#include <mj_message.h>

/** Numeric message template class. 

Note that this class contains method \b messageType that needs to be
specialized in child classes.
*/

template <class TElem>
class MJ_MessageNumeric : public MJ_Message
{
public:
  ///
  MJ_MessageNumeric(TElem value = 0, long time = -1) 
    : MJ_Message(time), m_value(value) {}
  ///
  virtual ~MJ_MessageNumeric() {}

  virtual float  toFloat(bool *ok = 0);
  virtual double toDouble(bool *ok = 0);
  virtual int    toInt(bool *ok = 0);
  virtual long   toLong(bool *ok = 0);
  
  /// Returns the type of this message
  virtual MJ_MessageType messageType() const;
  /// Duplicate this message
  virtual MJ_Message *duplicate();
    
  virtual bool write2(DI_DataBuffer2 *b);
  virtual bool read2(DI_DataBuffer2 *b, int readFields);

  virtual std::string describe();

  inline const TElem &value() const { return m_value; }
  inline void setValue(TElem value) { m_value = value; }

  inline void add(TElem a)        { m_value += a; }
  inline void divide(TElem div)   { m_value /= div; }
  inline void multiply(TElem mul) { m_value *= mul; }
  inline void subtract(TElem s)   { m_value -= s; }
  
  static MJ_Message *createNew();

protected:

  ///
  TElem m_value;
};


#endif
