/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_filter_ap1.h>
#include <mj_math.h>

template <class TElem> 
void MJ_FilterAP1<TElem>::putMany
(const float * samples, unsigned nofSamples)
{
  float mem = m_mem;
  const float a1 = m_a1;

  for(unsigned i=0; i < nofSamples; i++)
    mem = samples[i] - mem * a1;
  
  m_mem = mem;
}

template <class TElem> 
void MJ_FilterAP1<TElem>::putMany
(const double * samples, unsigned nofSamples)
{
  double mem = m_mem; // Local variable -> less memory accesses.
  const double a1 = m_a1;

  for(unsigned i=0; i < nofSamples; i++)
    mem = samples[i] - mem * a1;
  
  m_mem = mem;
}

template <class TElem> 
void MJ_FilterAP1<TElem>::putMany
(float * outsamples, const float * insamples, 
 unsigned nofSamples)
{
  double mem = m_mem; // Local variable -> less memory accesses.
  const double a1 = m_a1, b0 = m_b0;

  for(unsigned i=0; i < nofSamples; i++) {
    mem = insamples[i] - mem * a1;
    outsamples[i] = mem * b0;
  }
  m_mem = mem;
}


template <class TElem> 
void MJ_FilterAP1<TElem>::putManySquared
(const float * samples, unsigned nofSamples)
{
  float mem = m_mem;
  const float a1 = m_a1;
  const float *endBuf = & samples[nofSamples];

  while(samples < endBuf) {
    mem = *samples * *samples - mem * a1;
    samples++;
  }
  
  m_mem = mem;
}

template <class TElem> 
void MJ_FilterAP1<TElem>::designLowPass(TElem cutOffFreq)
{
  if((cutOffFreq >= 1.0) || (cutOffFreq <= 0.0)) return;

  const TElem fsin = MJ_sin(MJ_pi * cutOffFreq);
  const TElem fcos = MJ_cos(MJ_pi * cutOffFreq);

  const TElem x = 2.0;
  
  const TElem a1 = -1.0;

  const TElem b1 = x * 2.0 - 2.0 * fcos;
  const TElem c1 = fcos * fcos + fsin * fsin - x;
  const TElem d = MJ_sqrt(b1 * b1 - 4 * a1 *c1);
  const TElem g = (b1 - d) / (-2 * a1);

  m_b0 = (TElem) (1.0 - g);
  m_a1 = (TElem) (-g);

  /* cout << "MJ_FilterAP1::designLowPass " << cutOffFreq 
     << " b: " << m_b0 << " a: " << m_a1 << endl; */
}
