/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */
#include <mj_random.h>

#include <stdlib.h>
#ifndef WIN32
#include <sys/time.h>
#else
#include <windows.h>
#include <mmsystem.h>
#endif

unsigned MJ_Random::m_randMul=134695621;

static unsigned MJ_RandCount = 0;

MJ_Random MJ_Random::instance;

#ifndef WIN32
MJ_Random::MJ_Random()
  : m_val(++MJ_RandCount + (ulong) random())
{}
#else
MJ_Random::MJ_Random()
  : m_val(++MJ_RandCount + (ulong) rand())
{}
#endif

void MJ_Random::getManySamples0
(MJ_Sample *data, unsigned count, MJ_Sample width)
{
  MJ_Sample *endData = &data[count];
  while(data < endData) {
#if MJ_SAMPLE_IS_FLOAT == 1
    *data++ = getFloat0() * width;
#else
    *data++ = getDouble0() * width;
#endif
  }
}

void MJ_Random::getManySamples1
(MJ_Sample *data, unsigned count, MJ_Sample width)
{
  MJ_Sample *endData = &data[count];
  while(data < endData) {
#if MJ_SAMPLE_IS_FLOAT == 1
    *data++ = getFloat1(width);
#else
    *data++ = getDouble1(width);
#endif
  }
}

void MJ_Random::getTimeSeed()
{
  struct timeval tp;
#ifndef WIN32
  gettimeofday(&tp, 0);
#else
  int timeMs=timeGetTime();
  tp.tv_usec=timeMs*1000;
  tp.tv_sec=timeMs/1000;
#endif
  m_val = (unsigned) tp.tv_sec + (unsigned) tp.tv_usec - (++MJ_RandCount);
}
