#import "ParticleEmitterPatch.h"
#import "ParticleEmitterPatchUI.h"

#import "vee_emitter.h"
#import "vee_value_random.h"
#import "vee_value_operator.h"

@implementation ParticleEmitterPatch

- (id)attributes
{
	NSMutableDictionary *at=[[super attributes] mutableCopy];
	[at setObject:@"Kineme Particle Emitter" forKey:@"name"];
	return at;
}
+ (Class)inspectorClassWithIdentifier:(id)fp8
{
	return [ParticleEmitterPatchUI class];
}


- (id)initWithIdentifier:(id)fp8
{
//	NSLog(@"ParticleEmitterPatch::initWithIdentifier");
	self=[super initWithIdentifier:fp8];
	if(self)
	{
//		[inputEmit setBooleanValue:TRUE];

		[inputUpdateVelocity setBooleanValue:NO];

		[inputRate setDoubleValue:10.0];
		[inputRate setMinDoubleValue:0.0];

		[inputMaxCount setIndexValue:1e9];

		_childOperatorArrayCount=0;
		[self addChildOperator];

		[inputColor setRed:1.0 green:0.8 blue:0.8 alpha:0.5];
	}
	return self;
}


- (BOOL)setup:(QCOpenGLContext *)context
{
	[super setup:context];

//	NSLog(@"ParticleEmitterPatch::setup");

	int i;
	for(i=0;i<_childOperatorArrayCount;++i)
		_childOperatorArrayOld[i]=NULL;

	VEE_Emitter *e=new VEE_Emitter();
	[_object setObject:e];

	_emittedObject=NULL;

	return YES;
}
- (void)cleanup:(QCOpenGLContext *)context
{
	[super cleanup:context];

//	NSLog(@"ParticleEmitterPatch::cleanup");
	// _object takes care of releasing our VEE_Vortex

	// _emittedObject doesn't belong to us
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	[super execute:context time:time arguments:arguments];

	VEE_Emitter *e=(VEE_Emitter *)[_object object];

	VEE_Object *emittedObject=[[inputEmittedObject objectValue] object];
	if(emittedObject != _emittedObject)
	{
		_emittedObject=emittedObject;
//		NSLog(@"ParticleEmitterPatch::execute  Setting Emitted Object to %08x",_emittedObject);
		e->setType(_emittedObject);
	}

	int i;
	for(i=0;i<_childOperatorArrayCount;++i)
	{
		VEE_ValueOperator *op=[[_childOperatorArray[i] operatorValue] op];
		if(op != _childOperatorArrayOld[i])
		{
			if(_childOperatorArrayOld[i])
			{
//				NSLog(@"ParticleEmitterPatch::execute  Removing Child Operator[%d] %08x",i+1,_childOperatorArrayOld[i]);
				e->removeOperator
				(
					[[NSString stringWithFormat:@"Child Operator %i",i+1] cStringUsingEncoding:NSASCIIStringEncoding],
					false
				);
			}
			if(op)
			{
//				NSLog(@"ParticleEmitterPatch::execute  Adding Child Operator[%d] %08x",i+1,op);
				e->appendOperator2
				(
					op,
					[[NSString stringWithFormat:@"Child Operator %i",i+1] cStringUsingEncoding:NSASCIIStringEncoding]
				);
			}
			_childOperatorArrayOld[i]=op;
		}
	}

//	if( [inputEmit wasUpdated] )
//		e->setValueInt( "emit", [inputEmit booleanValue] );

	if( [inputRate wasUpdated] )
		e->setRate( [inputRate doubleValue] );

	if( [inputMaxCount wasUpdated] )
		e->setValueInt( "maxcount", [inputMaxCount indexValue] );

//	if( [inputAccumulate wasUpdated] )
//		e->setAccumulate( [inputAccumulate booleanValue] );

	if( [inputInheritVelocity wasUpdated] )
		e->setInheritVelocity( [inputInheritVelocity booleanValue] );

//	if( [inputCollisionRate wasUpdated] )
//		e->setValueFloat("collisionrate",[inputCollisionRate doubleValue]);

//	if( [inputCollisionNewRate wasUpdated] )
//		e->setValueFloat("collisionnewrate",[inputCollisionNewRate doubleValue]);

	return YES;
}




























- (void)addChildOperator
{
	{
		_childOperatorArray = (ParticleOperatorPort **) realloc
			(
				_childOperatorArray,
				((sizeof(ParticleOperatorPort*)) * (_childOperatorArrayCount+1))
			);
				
		_childOperatorArray[_childOperatorArrayCount] =
			[self createInputWithPortClass:[ParticleOperatorPort class] forKey:[NSString stringWithFormat:@"Child Operator %i",_childOperatorArrayCount+1] attributes:nil];

		[_childOperatorArray[_childOperatorArrayCount] retain];
	}

	{
		_childOperatorArrayOld = (VEE_ValueOperator **) realloc
			(
				_childOperatorArrayOld,
				((sizeof(VEE_ValueOperator*)) * (_childOperatorArrayCount+1))
			);

		_childOperatorArrayOld[_childOperatorArrayCount]=NULL;
	}

	++_childOperatorArrayCount;
}

- (void)removeChildOperator
{
	if(_childOperatorArrayCount>1)
	{
		{
			_childOperatorArray = (ParticleOperatorPort **) realloc
				(
					_childOperatorArray,
					((sizeof(ParticleOperatorPort*)) * (_childOperatorArrayCount-1))
				);
			
			[self deleteInputForKey:[NSString stringWithFormat:@"Child Operator %i",_childOperatorArrayCount]];

			[_childOperatorArray[_childOperatorArrayCount-1] release];
		}

		{
			if(_childOperatorArrayOld[_childOperatorArrayCount-1])
			{
//				NSLog(@"ParticleEmitterPatch::removeChildOperator  Removing Child Operator[%d] %08x",_childOperatorArrayCount,_childOperatorArrayOld[_childOperatorArrayCount-1]);
				VEE_Emitter *e=(VEE_Emitter *)[_object object];
				e->removeOperator
				(
					[[NSString stringWithFormat:@"Child Operator %i",_childOperatorArrayCount] cStringUsingEncoding:NSASCIIStringEncoding],
					false
				);
			}
			_childOperatorArrayOld = (VEE_ValueOperator **) realloc
				(
					_childOperatorArrayOld,
					((sizeof(VEE_ValueOperator*)) * (_childOperatorArrayCount-1))
				);
		}
		
		--_childOperatorArrayCount;
	}
}

- (NSDictionary*)state
{
	NSMutableDictionary *stateDict = [[NSMutableDictionary alloc] initWithCapacity:2];
	[stateDict addEntriesFromDictionary:[super state]];
	
	[stateDict setObject:[NSNumber numberWithInt:_childOperatorArrayCount] forKey:@"childOperatorArrayCount"];

	[stateDict autorelease];
	return stateDict;
}

- (BOOL)setState:(NSDictionary*)state
{
	int i;

	// clean up existing state, if it exists (init creates some)
	for(i=0;i<_childOperatorArrayCount;++i)
		[self deleteInputForKey:[NSString stringWithFormat:@"Child Operator %i",i+1]];
	
	_childOperatorArrayCount = [[state objectForKey:@"childOperatorArrayCount"] intValue];

	_childOperatorArray = (ParticleOperatorPort **) realloc
		(
			_childOperatorArray,
			((sizeof(ParticleOperatorPort*)) * (_childOperatorArrayCount))
		);

	for(i=0;i<_childOperatorArrayCount;++i)
	{
		_childOperatorArray[i] =
			[self createInputWithPortClass:[ParticleOperatorPort class] forKey:[NSString stringWithFormat:@"Child Operator %i",i+1] attributes:nil];

		[_childOperatorArray[i] retain];
	}

	return [super setState:state];
}

@end
