#import "ParticleObjectPatchAbstract.h"
#import "ParticleObjectPatchAbstractUI.h"

#import "vee_particle2.h"


@implementation ParticleObjectPatchAbstract

+ (int)executionModeWithIdentifier:(id)fp8
{
	return 0;
}
+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}
+ (int)timeModeWithIdentifier:(id)fp8
{
	return 0;
}
+ (Class)inspectorClassWithIdentifier:(id)fp8
{
	return [ParticleObjectPatchAbstractUI class];
}


- (id)initWithIdentifier:(id)fp8
{
//	NSLog(@"ParticleObjectPatchAbstract::initWithIdentifier");
	self=[super initWithIdentifier:fp8];
	if(self)
	{
		[inputEnable setBooleanValue:YES];

		[inputAgeLimit setDoubleValue:1e10];
		[inputAgeLimit setMinDoubleValue:0.0];

		_operatorArrayCount=0;
		[self addOperator];
	}
	return self;
}


- (BOOL)setup:(QCOpenGLContext *)context
{
//	NSLog(@"ParticleObjectPatchAbstract::setup");

	int i;
	for(i=0;i<_operatorArrayCount;++i)
		_operatorArrayOld[i]=NULL;

	_object=[[ParticleObject alloc] init];
	[outputObject setObjectValue:_object];

	return YES;
}

- (void)cleanup:(QCOpenGLContext *)context
{
//	NSLog(@"ParticleObjectPatchAbstract::cleanup");
	
	[_object release];
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
//	NSLog(@"ParticleObjectPatchAbstract::execute");

	VEE_Particle2 *o=(VEE_Particle2 *)[_object object];

	int i;
	for(i=0;i<_operatorArrayCount;++i)
	{
		VEE_ValueOperator *op=[[_operatorArray[i] operatorValue] op];
		if(op != _operatorArrayOld[i])
		{
			if(_operatorArrayOld[i])
			{
//				NSLog(@"ParticleObjectPatchAbstract::execute  Removing SelfOperator[%d] %08x",i+1,_operatorArrayOld[i]);
				o->removeSelfOperator
				(
					[[NSString stringWithFormat:@"Self Operator %i",i+1] cStringUsingEncoding:NSASCIIStringEncoding],
					false
				);
			}
			if(op)
			{
//				NSLog(@"ParticleObjectPatchAbstract::execute  Adding Self Operator[%d] %08x",i+1,op);
				o->appendSelfOperator2
				(
					op,
					[[NSString stringWithFormat:@"Self Operator %i",i+1] cStringUsingEncoding:NSASCIIStringEncoding]
				);
			}
			_operatorArrayOld[i]=op;
		}
	}

	if( [inputEnable wasUpdated] )
		o->setDone( ![inputEnable booleanValue] );

	if( [inputAgeLimit wasUpdated] )
		o->setAgeLimit( [inputAgeLimit doubleValue] );

	return YES;
}


- (void)addOperator
{
	{
		_operatorArray = (ParticleOperatorPort **) realloc
			(
				_operatorArray,
				((sizeof(ParticleOperatorPort*)) * (_operatorArrayCount+1))
			);
				
		_operatorArray[_operatorArrayCount] =
			[self createInputWithPortClass:[ParticleOperatorPort class] forKey:[NSString stringWithFormat:@"Self Operator %i",_operatorArrayCount+1] attributes:nil];

		[_operatorArray[_operatorArrayCount] retain];
	}

	{
		_operatorArrayOld = (VEE_ValueOperator **) realloc
			(
				_operatorArrayOld,
				((sizeof(VEE_ValueOperator*)) * (_operatorArrayCount+1))
			);

		_operatorArrayOld[_operatorArrayCount]=NULL;
	}

	++_operatorArrayCount;
}

- (void)removeOperator
{
	if(_operatorArrayCount>1)
	{
		{
			_operatorArray = (ParticleOperatorPort **) realloc
				(
					_operatorArray,
					((sizeof(ParticleOperatorPort*)) * (_operatorArrayCount-1))
				);
			
			[self deleteInputForKey:[NSString stringWithFormat:@"Self Operator %i",_operatorArrayCount]];

			[_operatorArray[_operatorArrayCount-1] release];
		}

		{
			if(_operatorArrayOld[_operatorArrayCount-1])
			{
//				NSLog(@"ParticleObjectPatchAbstract::removeOperator  Removing SelfOperator[%d] %08x",_operatorArrayCount,_operatorArrayOld[_operatorArrayCount-1]);
				VEE_Particle2 *o=(VEE_Particle2 *)[_object object];
				o->removeSelfOperator
				(
					[[NSString stringWithFormat:@"Self Operator %i",_operatorArrayCount] cStringUsingEncoding:NSASCIIStringEncoding],
					false
				);
			}
			_operatorArrayOld = (VEE_ValueOperator **) realloc
				(
					_operatorArrayOld,
					((sizeof(VEE_ValueOperator*)) * (_operatorArrayCount-1))
				);
		}
		
		--_operatorArrayCount;
	}
}


- (NSDictionary*)state
{
	NSMutableDictionary *stateDict = [[NSMutableDictionary alloc] initWithCapacity:2];
	[stateDict addEntriesFromDictionary:[super state]];
	
	[stateDict setObject:[NSNumber numberWithInt:_operatorArrayCount] forKey:@"operatorArrayCount"];

	[stateDict autorelease];
	return stateDict;
}
- (BOOL)setState:(NSDictionary*)state
{
	int i;

	// clean up existing state, if it exists (init creates some)
	for(i=0;i<_operatorArrayCount;++i)
		[self deleteInputForKey:[NSString stringWithFormat:@"Self Operator %i",i+1]];
	
	_operatorArrayCount = [[state objectForKey:@"operatorArrayCount"] intValue];

	_operatorArray = (ParticleOperatorPort **) realloc
		(
			_operatorArray,
			((sizeof(ParticleOperatorPort*)) * (_operatorArrayCount))
		);

	for(i=0;i<_operatorArrayCount;++i)
	{
		_operatorArray[i] =
			[self createInputWithPortClass:[ParticleOperatorPort class] forKey:[NSString stringWithFormat:@"Self Operator %i",i+1] attributes:nil];

		[_operatorArray[i] retain];
	}

	return [super setState:state];
}

@end
