#import "ParticleScenePatch.h"
#import "ParticleScenePatchUI.h"

#import "OpenGL/gl.h"
#import "OpenGL/OpenGL.h"



@implementation ParticleScenePatch : QCPatch

+ (int)executionModeWithIdentifier:(id)fp8
{
	return 0;
}
+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}
+ (int)timeModeWithIdentifier:(id)fp8
{
	return 1;
}
- (id)attributes
{
	NSMutableDictionary *at=[[super attributes] mutableCopy];
	[at setObject:@"Kineme Particle Scene" forKey:@"name"];
	return at;
}
+ (Class)inspectorClassWithIdentifier:(id)fp8
{
	return [ParticleScenePatchUI class];
}


- (id)initWithIdentifier:(id)fp8
{
//	NSLog(@"ParticleScenePatch::initWithIdentifier");
	self=[super initWithIdentifier:fp8];
	if(self)
	{
		[inputSplitThreshold setIndexValue:30];
		[inputCollapseThreshold setIndexValue:20];
		[inputAllowRecursionLevels setIndexValue:1];

		[inputBoundLowerX setDoubleValue:-10.0];
		[inputBoundLowerY setDoubleValue:-10.0];
		[inputBoundLowerZ setDoubleValue:-10.0];
		[inputBoundUpperX setDoubleValue:10.0];
		[inputBoundUpperY setDoubleValue:10.0];
		[inputBoundUpperZ setDoubleValue:10.0];

		_objectArrayCount=0;
		[self addObject];
	}
	return self;
}


- (BOOL)setup:(QCOpenGLContext *)context
{
//	NSLog(@"ParticleScenePatch::setup");

	int i;
	for(i=0;i<_objectArrayCount;++i)
		_objectArrayOld[i]=NULL;

	_scenebox=new VEE_SceneBox();
	_scene=[[ParticleScene alloc] initWithScene:_scenebox];
	[outputScene setSceneValue:_scene];

//	geeEngine=new GEE_Engine();
//	geeEngine->rewind(0);

	_lastFrameTime=0.0;

	return YES;
}
- (void)cleanup:(QCOpenGLContext *)context
{
//	NSLog(@"ParticleScenePatch::cleanup");
	
	[_scene release];
	// _scene takes care of releasing our VEE_SceneBox
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	if( [inputAllowRecursionLevels indexValue] < 1 )
		[inputAllowRecursionLevels setIndexValue:1];


	int i;
	for(i=0;i<_objectArrayCount;++i)
	{
		VEE_Object *obj=[[_objectArray[i] objectValue] object];
		if(obj != _objectArrayOld[i])
		{
			if(_objectArrayOld[i])
			{
//				NSLog(@"ParticleScenePatch::execute  Removing Object[%d] %08x",i+1,_objectArrayOld[i]);
				_scenebox->removeObject(_objectArrayOld[i]);
			}
			if(obj)
			{
//				NSLog(@"ParticleScenePatch::execute  Adding Object[%d] %08x (%s)",i+1,obj,obj?typeid(*obj).name():"(null)");
				_scenebox->addNewObject(obj);
			}
			_objectArrayOld[i]=obj;
		}
	}




	if(
		[inputBoundLowerX wasUpdated]
		|| [inputBoundLowerY wasUpdated]
		|| [inputBoundLowerZ wasUpdated]
		|| [inputBoundUpperX wasUpdated]
		|| [inputBoundUpperY wasUpdated]
		|| [inputBoundUpperZ wasUpdated]
	)
		_scenebox->setArea
		(
			VEE_Vector3
			(
				[inputBoundLowerX doubleValue],
				[inputBoundLowerY doubleValue],
				[inputBoundLowerZ doubleValue]
			),
			VEE_Vector3
			(
				[inputBoundUpperX doubleValue],
				[inputBoundUpperY doubleValue],
				[inputBoundUpperZ doubleValue]
			)
		);



		

	// do the calculus

//		NSLog(@"ParticleScenePatch::execute  _scenebox->update()");

		_scenebox->update(
						[inputSplitThreshold indexValue],
						[inputCollapseThreshold indexValue],
						[inputAllowRecursionLevels indexValue],
						time - _lastFrameTime
					);

//		{
//			GEE_TimeStamp ts;
//			ts.setSeconds(time);
//			geeEngine->update(ts);
//		}

		[outputObjectCount setIndexValue:_scenebox->totalChildren()];
		[outputForceCount setIndexValue:_scenebox->totalForces()];

	_lastFrameTime=time;

	return YES;
}

- (void)addObject
{
	{
		_objectArray = (ParticleObjectPort **) realloc
			(
				_objectArray,
				((sizeof(ParticleObjectPort*)) * (_objectArrayCount+1))
			);
				
		_objectArray[_objectArrayCount] =
			[self createInputWithPortClass:[ParticleObjectPort class] forKey:[NSString stringWithFormat:@"Object %i",_objectArrayCount+1] attributes:nil];

		[_objectArray[_objectArrayCount] retain];
	}

	{
		_objectArrayOld = (VEE_Object **) realloc
			(
				_objectArrayOld,
				((sizeof(VEE_Object*)) * (_objectArrayCount+1))
			);

		_objectArrayOld[_objectArrayCount]=NULL;
	}

	++_objectArrayCount;
}
- (void)removeObject
{
	if(_objectArrayCount>1)
	{
		{
			_objectArray = (ParticleObjectPort **) realloc
				(
					_objectArray,
					((sizeof(ParticleObjectPort*)) * (_objectArrayCount-1))
				);
			
			[self deleteInputForKey:[NSString stringWithFormat:@"Object %i",_objectArrayCount]];

			[_objectArray[_objectArrayCount-1] release];
		}

		{
			if(_objectArrayOld[_objectArrayCount-1])
			{
//				NSLog(@"ParticleScenePatch::removeObject  Removing Object[%d] %08x",_objectArrayCount,_objectArrayOld[_objectArrayCount-1]);
				_scenebox->removeObject(_objectArrayOld[_objectArrayCount-1]);
			}
			_objectArrayOld = (VEE_Object **) realloc
				(
					_objectArrayOld,
					((sizeof(VEE_Object*)) * (_objectArrayCount-1))
				);
		}
		
		--_objectArrayCount;
	}
}

- (NSDictionary*)state
{
	NSMutableDictionary *stateDict = [[NSMutableDictionary alloc] initWithCapacity:2];
	[stateDict addEntriesFromDictionary:[super state]];
	
	[stateDict setObject:[NSNumber numberWithInt:_objectArrayCount] forKey:@"objectArrayCount"];

	[stateDict autorelease];
	return stateDict;
}

- (BOOL)setState:(NSDictionary*)state
{
	int i;

	// clean up existing state, if it exists (init creates some)
	for(i=0;i<_objectArrayCount;++i)
		[self deleteInputForKey:[NSString stringWithFormat:@"Object %i",i+1]];
	
	_objectArrayCount = [[state objectForKey:@"objectArrayCount"] intValue];

	_objectArray = (ParticleObjectPort **) realloc
		(
			_objectArray,
			((sizeof(ParticleObjectPort*)) * (_objectArrayCount))
		);

	for(i=0;i<_objectArrayCount;++i)
	{
		_objectArray[i] =
			[self createInputWithPortClass:[ParticleObjectPort class] forKey:[NSString stringWithFormat:@"Object %i",i+1] attributes:nil];

		[_objectArray[i] retain];
	}

	{
		_objectArrayOld = (VEE_Object **) realloc
			(
				_objectArrayOld,
				((sizeof(VEE_Object*)) * (_objectArrayCount))
			);
		for(i=0;i<_objectArrayCount;++i)
			_objectArrayOld[i]=NULL;
	}

	return [super setState:state];
}

@end
