/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */
#ifndef VECTOR_HPP
#define VECTOR_HPP

namespace Fluid {

  /**
   * Well well... it's a vector
   *
   * It's written because I thought std::vector doesn't work. It was
   * my bug however. But the Vector remained :)
   */

template <class T>
class Vector
{
public:
  Vector(int n)
    :n(n)
  {
    assert(n > 0);
    arr = new T[n];
  }
  ~Vector()
  {
    delete [] arr;
  }

  T& operator[] (int index) { assert(index >= 0 && index < n); return arr[index]; }
  const T& operator[] (int index) const { assert(index >= 0 && index < n); return arr[index]; }

  int size() const { return n; }

  void resize(int newsize)
  {
    assert(newsize > 0);
    if(newsize != n)
      {
	T* tmp = arr;
	arr = new T[newsize];
	int min = newsize < n ? newsize : n;
	for(int i = 0; i < min; i++)
	  {
	    arr[i] = tmp[i];
	  }
	delete [] tmp;
	n = newsize;
      }
  }
private:
  Vector(const Vector<T>& that) {}
  Vector<T>& operator=(const Vector<T>& that) { return *this; }
  T *arr;
  int n;
};

} // namespace 

#endif
