/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_collector.h>

#include <vee_error.h>
#include <vee_force_object.h>
#include <vee_scene.h>

  
VEE_Collector::VEE_Collector()
  : m_collectedObjects(0)
{
  m_objects.resize(5000);
}

VEE_Collector::~VEE_Collector()
{
  clear();
}

void VEE_Collector::addScene(VEE_SceneBox *scene)
{
  // VEE_trace("VEE_Collector::addScene # %p %u", scene, scene->numObjects());

  VEE_SceneBox::iterator it = scene->begin();
  VEE_SceneBox::iterator sentinel = scene->end();
  
  while(it != sentinel) {

    VEE_Object * o = VEE_SceneBox::getObject(it);

    int type = o->objectType();

    if(type == VEE_OBJECT_PARTICLE) {
      VEE_Particle * p = (VEE_Particle *) o;
      
      if(p->visible())
	m_objects[m_collectedObjects++] = *o;

      if(m_collectedObjects >= m_objects.size())
	reserve(m_collectedObjects + 1000);

      /* const VEE_Vector3 & loc = o->location();
      float distance = (m_camPos - loc).length();
      m_objects.insert(std::pair<float, VEE_ParticleVisualInfo>(distance, *o));
      */
    }
    else if(type == VEE_OBJECT_FORCE) {

      VEE_ForceObject *force = (VEE_ForceObject *) o;

      if(!force->renderAsForce()) {
	
	m_objects[m_collectedObjects++] = *o;

	if(m_collectedObjects >= m_objects.size())
	  reserve(m_collectedObjects + 1000);
      }
      else {
	VEE_Object *o2 = o->clone();
	
	VEE_ForceObject *fo = dynamic_cast<VEE_ForceObject *> (o2);
	
	if(!fo)
	  VEE_fatal("VEE_OrdererCollector::addScene # Not force %s (%s)",
		    typeid(*o).name(), typeid(*o2).name());
	
	if(fo->sceneBox())
	  fo->sceneBox()->removeChildObject(o2);
	
	fo->setSceneBox(0);
	fo->forgetLinks();
      
	m_forces.push_back(fo);
      }
    }
    it++;
  }

  if(scene->isSplit()) {
    for(uint j = 0; j < VEE_SB_CHILDREN; j++) 
      addScene(scene->child(j));
  }  
}

/** Clears the collected elements. Note that this method does *not*
    unallocate the memory used to store the normal objects. It simply
    sets the counters to zero. It DOES delete the force-objects that
    were around. */

void VEE_Collector::clear()
{
  m_collectedObjects = 0;

  for(forceIterator it = forceBegin(); it != forceEnd(); it++) {
    VEE_ForceObject * force = getForce(it);
    /* VEE_trace("VEE_OrdererCollector::clear # About to delete %s %p",
       typeid(*force).name(), force); */
    delete force;
  }

  m_forces.clear();
}
