/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_COLLECTOR_H
#define VEE_COLLECTOR_H

#include <vee_object.h>

#include <list>
#include <vector>

class VEE_ForceObject;

class VEE_Collector
{
public:
  typedef std::vector<VEE_ParticleVisualInfo>::iterator iterator;
  typedef std::list<VEE_ForceObject *>::iterator forceIterator;
  
  VEE_Collector();
  ~VEE_Collector();
  
 
  /// Gets the first object
  iterator begin() { return m_objects.begin();}
  uint collectedObjects() const { return m_collectedObjects; }

  /// Gets the first object
  forceIterator forceBegin() { return m_forces.begin();}
  /// Gets the last object (one past the end)
  forceIterator forceEnd() { return m_forces.end(); }
  
  uint forceCount() { return m_forces.size(); }

  /// Gets the object from the iterator
  static VEE_ParticleVisualInfo & getObject(iterator & it)
  { return (*it); }

  /// Gets the force from the force iterator
  static VEE_ForceObject * getForce(forceIterator & it)
  { return (*it); }
 
  void addScene(VEE_SceneBox *);
  void clear();
  void reserve(uint n) { if(n > m_objects.size()) m_objects.resize(n); }

protected:
  std::vector<VEE_ParticleVisualInfo> m_objects;
  std::list<VEE_ForceObject *>        m_forces;
  
  uint                                m_collectedObjects;

};

#endif
