/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_HAS_VARIABLES_H
#define VEE_HAS_VARIABLES_H

#include <vee_variable.h>

class VEE_HasVariables
{
public:
  VEE_HasVariables() {}
  virtual ~VEE_HasVariables() {}

  virtual VEE_Variable getVariable(const char *) const;

  inline float getVariableFloat(const char *, float def = 0.0) const;
  inline int   getVariableInt(const char *, int def = 0.0) const;
  inline VEE_Rgba getVariableRgba(const char *, const VEE_Rgba & def = VEE_Rgba::White) const;
  inline VEE_Vector3 getVariableVector3(const char *, const VEE_Vector3 & def = VEE_Vector3ZERO) const;
  inline VEE_Matrix3 getVariableMatrix3(const char *, const VEE_Matrix3 & def = VEE_Matrix3IDENTITY) const;

};


/// Shortcut to get a float
float VEE_HasVariables::getVariableFloat(const char *id, float def) const
{
  VEE_Variable v = getVariable(id);
  return v.getFloat(def);
}

/// Shortcut to get an integer 
int VEE_HasVariables::getVariableInt(const char *id, int def) const
{
  VEE_Variable v = getVariable(id);
  return v.getInt(def);
}

/// Shortcut to get an RGBA color 
VEE_Rgba VEE_HasVariables::getVariableRgba
(const char *id, const VEE_Rgba & def) const
{
  VEE_Variable v = getVariable(id);
  return v.getRgba(def);
}

/// Shortcut to get a vector3
VEE_Vector3 VEE_HasVariables::getVariableVector3
(const char *id, const VEE_Vector3 & def) const
{
  VEE_Variable v = getVariable(id);
  return v.getVector3(def);
}

/// Shortcut to get a matrix3
VEE_Matrix3 VEE_HasVariables::getVariableMatrix3
(const char *id, const VEE_Matrix3 & def) const
{
  VEE_Variable v = getVariable(id);
  return v.getMatrix3(def);
}

#endif
