/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_LOOKS_COLOR_TABLE_H
#define VEE_LOOKS_COLOR_TABLE_H

#include <vee_looks.h>

#include <vee_color_table.h>
#include <vee_reference_object.h>

class VEE_LooksColorTable : public VEE_Looks
{
public:
  VEE_LooksColorTable() {}
  VEE_LooksColorTable(VEE_RgbaTable *colorTable)
    : m_colorTable(*colorTable) {}

  virtual ~VEE_LooksColorTable();

  virtual VEE_Rgba color();
  virtual VEE_Looks *clone() const;
  
  VEE_RgbaTable & colorTable() { return *m_colorTable; }

protected:

  VEE_ReferenceObject<VEE_RgbaTable> m_colorTable;
};


/** A color table look-up with matching to given maximum age. */
class VEE_LooksColorTable2 : public VEE_Looks
{
public:
  VEE_LooksColorTable2() : m_maxAge(1.0) {}
  VEE_LooksColorTable2(VEE_RgbaTable *colorTable, float maxAge = 1.0)
    : m_maxAge(maxAge), m_colorTable(*colorTable) {}

  virtual ~VEE_LooksColorTable2();

  virtual VEE_Rgba color();
  virtual VEE_Looks *clone() const;
  
  VEE_RgbaTable & colorTable() { return *m_colorTable; }

  void setMaxAge(float max) { m_maxAge = max; }

protected:

  float m_maxAge;

  VEE_ReferenceObject<VEE_RgbaTable> m_colorTable;
};



#endif
