/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_MARCHING_CUBES_H
#define VEE_MARCHING_CUBES_H

#include <vee_vector3.h>

#include <mj_array3d.h>

#include <vector>

class VEE_VolumeGrid2;

class VEE_MarchingCubes
{
public:
  class Triangle
  {
  public:
    VEE_Vector3 m_vertices[3];
  };

  VEE_MarchingCubes();
  ~VEE_MarchingCubes();
  
  void triangulateGrid(const MJ_Array3D<float> &field, float threshold = 0.5);

  void triangulateSubGrid(VEE_VolumeGrid2 *grid, 
			  int xmin, int ymin, int zmin,
			  int xmax, int ymax, int zmax,
			  float threshold = 0.5);

  const Triangle & triangle(uint i) const { return m_triangles[i]; }
  uint triangleCount() const { return m_triCount; }

  void clear() { m_triCount = 0; }

protected:

  inline void addTri(const Triangle &tri)
  {
    if(m_triCount >= m_triangles.size())
      m_triangles.resize(m_triCount + 10);
    m_triangles[m_triCount++] = tri;
  }

  inline void handleCube(VEE_VolumeGrid2 *field, float threshold,
			 int x, int y, int z);

  inline void handleCube(const MJ_Array3D<float> &, float threshold,
			 uint x, uint y, uint z);

  std::vector<Triangle> m_triangles;
  uint m_triCount;
};

#endif
