
/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_meta_balls.h>

#include <vee_bounding_box.h>

VEE_MetaBalls::VEE_MetaBalls()
{}

VEE_MetaBalls::~VEE_MetaBalls()
{}

float VEE_MetaBalls::presence(const VEE_Vector3 &loc, float rfull)
{
  float r = 0.0;
  
  for(uint i=0; i < m_balls.size(); i++) {
    float p = m_balls[i].presence(loc, rfull);

    if(r < p)
      r = p; 
    // r += p;
  }

  return r;
}


void VEE_MetaBalls::getBoundingBox(VEE_BoundingBox *bb) const
{
  if(!m_balls.size()) {
    bb->clear();
    return;
  }
  
  bb->clear(m_balls[0].m_center);

  for(uint i=0; i < m_balls.size(); i++) {
    const Ball & ball = m_balls[i];

    VEE_Vector3 vec1 = ball.m_center;
    VEE_Vector3 vec2;
    vec2.make(ball.m_radius);

    bb->expand(vec1 + vec2);
    bb->expand(vec1 - vec2);
  }
  
}
