/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_object_factory.h>

#include <vee_error.h>

#include <vee_blow.h>
#include <vee_curve_particle.h>
#include <vee_damping.h>
#include <vee_explosion.h>
#include <vee_force_plane.h>
#include <vee_gravitate.h>
#include <vee_gravity.h>
#include <vee_particle.h>
#include <vee_random_force.h>
// #include <vee_rigid_particle.h>
#include <vee_vortex.h>
#include <vee_tail_particle.h>
#include <vee_tube.h>
#include <vee_wind.h>

#include <map>
#include <string>

typedef std::map<std::string, VEE_Object *(*)()> factoryContainer;
static factoryContainer m_factories;

// typedef std::map<std::string, VEE_Object *(*)()>::iterator objIter;

VEE_Object * VEE_ObjectFactory::create(const char *type)
{
  // VEE_trace("VEE_ObjectFactory::create # \"%s\"", type);

  std::string tmp(type);

  factoryContainer::iterator it = 
    m_factories.find(tmp);

  if(it == m_factories.end()) {
    VEE_error("VEE_ObjectFactory::create # No factory for \"%s\"", type);
    return 0;
  }
  
  VEE_Object *(*factory)() = (*it).second;

  VEE_Object *obj = factory ? (*factory)() : 0;

  if(!obj) {
    VEE_error("VEE_ObjectFactory::create # Factory failed for for \"%s\"", 
	      type);    
  }

  return obj;
}
  
void VEE_ObjectFactory::addFactory(const char *type, VEE_Object *(*factory)())
{
  m_factories[type] = factory;
}

void VEE_ObjectFactory::init()
{
  addFactory("gravity",   VEE_Gravity::factory);
  addFactory("gravitate", VEE_Gravitate::factory);
  addFactory("blow",      VEE_Blow::factory);
  addFactory("curve",     VEE_CurveParticle::factory);
  addFactory("damping",   VEE_Damping::factory);
  addFactory("explosion", VEE_Explosion::factory);
  addFactory("forceplane",VEE_ForcePlane::factory);
  addFactory("particle",  VEE_Particle::factory);
  addFactory("randomforce",   VEE_RandomForce::factory);
  addFactory("vortex",        VEE_Vortex::factory);
  addFactory("tailparticle",  VEE_TailParticle::factory);
  addFactory("tailparticle2", VEE_TailParticle2::factory);
  addFactory("tailparticle3", VEE_TailParticle3::factory);
  addFactory("tube",      VEE_Tube::factory);
  addFactory("wind",      VEE_Wind::factory);
}
