/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */


#include <vee_operator_dt.h>

#include <vee_object.h>
#include <vee_strs.h>
#include <vee_value.h>

VEE_OperatorDt::~VEE_OperatorDt()
{}

/** Sets a value.

    <OL>

    <LI> "dt", float, time to use when adding dt, range 0 to inf

    <LI> "moveback", bool

    <LI> "allowagechange", bool

    </OL>

*/

bool VEE_OperatorDt::setValue(const char *id, VEE_Value *v)
{
  bool ok = false;

  if(strcmp(id, "dt") == 0) {
    float tmp;
    ok = v->getFloat(&tmp);
    if(ok)
      m_dt = tmp;
  }
  else if(strcmp(id, "moveback") == 0) {
    int tmp;
    ok = v->getInt(&tmp);
    if(ok)
      m_moveBack = tmp;
  }
  else if(strcmp(id, "resetage") == 0) {
    int tmp;
    ok = v->getInt(&tmp);
    if(ok)
      m_resetAge = tmp;
  }

  return ok;
}

void VEE_OperatorDt::operate(VEE_Object *o, float)
{
  VEE_Vector3 loc = o->location();
  float age = o->age();

  int div = (int) (m_dt / m_maxStep + 0.5f);
  if(div < 1) div = 1;

  float dt = m_dt / div;

  for(int i=0; i < div; i++) {
    o->updateVelocity(dt);
    o->update(dt);
  }

  // Optionally move the object back to the original location:
  if(m_moveBack)
    o->setValueVector3(VEE_Strs::location, loc);

  // Optionally set the age to what it was:
  if(m_resetAge) {
    o->setAge(age);
    // The age limit might have been passed accidentally, so return it
    o->setDone(o->age() > o->ageLimit());
  }
}

VEE_Operator * VEE_OperatorDt::clone() const
{
  return new VEE_OperatorDt(*this);
}
