/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_operator_flash.h>

#include <vee_object.h>
#include <vee_perlin_noise.h>
#include <vee_ptr.h>


VEE_OperatorFlash::VEE_OperatorFlash()
  : m_frequency(1.0),
    m_min(0.0),
    m_max(2.0)
{}

VEE_OperatorFlash::~VEE_OperatorFlash()
{}

void VEE_OperatorFlash::operate(VEE_Object *o, float)
{
  float age = o->age();
  float offset = VEE_Ptr::ptrToRandom(o) * 1000.0f;
  
  float noise = VEE_PerlinNoise::noise(age * m_frequency + offset);
  
  o->setColor(o->color() * (noise * (m_max - m_min) + m_min));
}

VEE_Operator * VEE_OperatorFlash::clone() const
{
  return new VEE_OperatorFlash(*this);
}
