/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_operator_math.h>

template <class T>
VEE_OperatorMath<T>::VEE_OperatorMath()
  : m_operation(VEE_MATH_ADD),
    m_value(defaultValue())
{}

template <class T>
VEE_OperatorMath<T>::~VEE_OperatorMath()
{}

template <>
void VEE_OperatorMath<float>::operate(VEE_Object *o, float)
{
  
  if(m_operation == VEE_MATH_NONE) {
    o->setValueFloat(m_id.c_str(), m_value);
    return;
  }
  
  float val = o->getVariableFloat(m_id.c_str());
  
  if(m_operation == VEE_MATH_ADD)
    val += m_value;
  if(m_operation == VEE_MATH_SUBTRACT)
    val -= m_value;
  if(m_operation == VEE_MATH_MULTIPLY)
    val *= m_value;
  if(m_operation == VEE_MATH_DIVIDE)
    val /= m_value;
  
  o->setValueFloat(m_id.c_str(), val);
}


template <class T>
VEE_Operator * VEE_OperatorMath<T>::clone() const
{
  return new VEE_OperatorMath<T>(*this);
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

template class VEE_OperatorMath<float>;
