/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_operator_random_tmpl.h>

#include <vee_error.h>
#include <vee_value.h>

VEE_OperatorRandomSphereVector::VEE_OperatorRandomSphereVector()
  : m_radius(1.0)
{}

VEE_OperatorRandomSphereVector::~VEE_OperatorRandomSphereVector()
{}

void VEE_OperatorRandomSphereVector::operate(VEE_Object * o, float)
{
  VEE_Vector3 r;

  for(int iterate = 1000; iterate > 0; iterate--) {
 
   r.make(VEE_rand.rand11(), VEE_rand.rand11(), VEE_rand.rand11());

    if(r.length() <= 1.0) {
      iterate = 0;
      r *= m_radius;
    }
  }

  o->setValueVector3(m_id.c_str(), r);
}

VEE_Operator * VEE_OperatorRandomSphereVector::clone() const
{
  return new VEE_OperatorRandomSphereVector(*this);
}

bool VEE_OperatorRandomSphereVector::setValue(const char *id, VEE_Value *v)
{
  bool ok = false;
  if(strcmp(id, "radius") == 0) {
    float val;
    ok = v->getFloat(&val);

    if(ok)
      m_radius = val;
  }
  return ok;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

VEE_OperatorRandomFloat::VEE_OperatorRandomFloat()
  : m_radius(1.0),
    m_operation(VEE_MATH_NONE)
{}
  
VEE_OperatorRandomFloat::~VEE_OperatorRandomFloat()
{}

void VEE_OperatorRandomFloat::operate(VEE_Object *o, float)
{
  float r = VEE_rand.rand0X(m_radius * 2.0) - m_radius;

  if(m_operation == VEE_MATH_NONE) {
    o->setValueFloat(m_id.c_str(), r);
    return;
  }
  
  float val = o->getVariableFloat(m_id.c_str());
  
  if(m_operation == VEE_MATH_ADD)
    val += r;
  if(m_operation == VEE_MATH_SUBTRACT)
    val -= r;
  if(m_operation == VEE_MATH_MULTIPLY)
    val *= r;
  if(m_operation == VEE_MATH_DIVIDE)
    val /= r;
  
  o->setValueFloat(m_id.c_str(), val);
}

VEE_Operator * VEE_OperatorRandomFloat::clone() const
{
  return new VEE_OperatorRandomFloat(*this);
}

bool VEE_OperatorRandomFloat::setValue(const char *id, VEE_Value *v)
{
  bool ok = false;

  if(strcmp(id, "radius") == 0) {
    float val;
    ok = v->getFloat(&val);

    if(ok) m_radius = val;
  }

  return ok;
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

VEE_OperatorPerlinVector::VEE_OperatorPerlinVector()
  : m_operation(VEE_MATH_NONE)
{
  m_scale.make(1.0f);
}

VEE_OperatorPerlinVector::~VEE_OperatorPerlinVector()
{}

void VEE_OperatorPerlinVector::operate(VEE_Object *o, float)
{
  VEE_trace("VEE_OperatorPerlinVector::operate # Unimplemented");
}

VEE_Operator * VEE_OperatorPerlinVector::clone() const
{
  return new VEE_OperatorPerlinVector(*this);
}

bool VEE_OperatorPerlinVector::setValue(const char *, VEE_Value *)
{
  return false;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

template class VEE_OperatorRandomSelect<int>;
template class VEE_OperatorRandomSelect<float>;
template class VEE_OperatorRandomSelect<VEE_Vector3>;
template class VEE_OperatorRandomSelect<VEE_Matrix3>;
