/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_OPERATOR_RANDOM_H
#define VEE_OPERATOR_RANDOM_H

#include <vee_object.h>
#include <vee_operator.h>

#include <vector>

enum VEE_MathOperation {
  VEE_MATH_NONE,
  VEE_MATH_ADD,
  VEE_MATH_SUBTRACT,
  VEE_MATH_MULTIPLY,
  VEE_MATH_DIVIDE,
  VEE_MATH_SIZEOF
};

template <class T>
class VEE_OperatorRandomSelect : public VEE_Operator
{
public:
  VEE_OperatorRandomSelect();
  virtual ~VEE_OperatorRandomSelect();
  virtual void operate(VEE_Object *, float);
  virtual VEE_Operator * clone() const;

  void addValue(T v);
  bool removeValue(T v);
  int  find(T v);

  void setId(const std::string &id) { m_id = id; }

  inline void deliver(VEE_Object *o, const T &);

protected:
  std::vector<T> m_values;
  std::string    m_id;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_OperatorRandomSphereVector : public VEE_Operator
{
public:
  VEE_OperatorRandomSphereVector();
  virtual ~VEE_OperatorRandomSphereVector();
  virtual void operate(VEE_Object *, float);
  virtual VEE_Operator * clone() const;

  virtual bool setValue(const char *, VEE_Value *);

  void setId(const std::string &id) { m_id = id; }
  void setRadius(float r) { m_radius = r; }

protected:
  std::string    m_id;
  float          m_radius;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_OperatorRandomFloat : public VEE_Operator
{
public:
  VEE_OperatorRandomFloat();
  virtual ~VEE_OperatorRandomFloat();
  virtual void operate(VEE_Object *, float);
  virtual VEE_Operator * clone() const;

  virtual bool setValue(const char *, VEE_Value *);

  void setId(const std::string &id) { m_id = id; }
  void setRadius(float r) { m_radius = r; }
  void setOperation(VEE_MathOperation op) { m_operation = op; }

protected:
  std::string    m_id;
  float          m_radius;
  VEE_MathOperation m_operation;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_OperatorPerlinVector : public VEE_Operator
{
public:
  VEE_OperatorPerlinVector();
  virtual ~VEE_OperatorPerlinVector();
  virtual void operate(VEE_Object *, float);
  virtual VEE_Operator * clone() const;

  virtual bool setValue(const char *, VEE_Value *);

  void setId(const std::string &id) { m_id = id; }
  void setScale(const VEE_Vector3 & s) { m_scale = s; }
  void setOperation(VEE_MathOperation op) { m_operation = op; }

protected:
  std::string       m_id;
  VEE_Vector3       m_scale;
  VEE_MathOperation m_operation;
};

#endif
