/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_PARTICLE2_H
#define VEE_PARTICLE2_H

#include <vee_particle.h>
#include <vee_operator_list.h>

/** A particle with self-operator list. */

class VEE_Particle2 : public VEE_Particle
{
public:
  VEE_Particle2();
  VEE_Particle2(const VEE_Vector3 &location,  const VEE_Vector3 &velocity, 
		float ageLimit = 1e+10)
    : VEE_Particle(location, velocity, ageLimit, 0) {}

  virtual ~VEE_Particle2();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Object *clone() const;
  virtual void update(float dt);

  void appendSelfOperator(VEE_Operator *o) { m_slist.appendOperator(o); }
  void appendSelfOperator2(VEE_Operator *o, const char * name) 
  { m_slist.appendOperator(o, name); }
  bool removeSelfOperator(const char *name, bool alsoDelete)
  { return m_slist.removeOperator(name, alsoDelete); }

protected:
  VEE_OperatorList m_slist;
};

#endif
