/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_TAIL_PARTICLE_H
#define VEE_TAIL_PARTICLE_H

#include <vee_particle.h>
#include <vee_math.h>

#include <mj_ringbuffer.h>

/** A particle that leaves tail. */
class VEE_TailParticle : public VEE_Particle
{
public:
  VEE_TailParticle();
  virtual ~VEE_TailParticle();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Variable getVariable(const char *) const;

  virtual void update(float);
  
  virtual VEE_Object *clone() const;
  static VEE_Object * factory();

  virtual void resizeTail(uint);

  uint pointsToRender() const
  { return VEE_min((uint) m_points.getSampleCount(), (uint) (m_nofPoints)); }
  
  class Point {
  public:
    
    VEE_Rgba & color() { return m_color; }
    const VEE_Rgba & color() const { return m_color; }

    VEE_Vector3 & location() { return m_location; }
    const VEE_Vector3 & location() const { return m_location; }

    VEE_Vector3 & velocity() { return m_velocity; }
    const VEE_Vector3 & velocity() const { return m_velocity; }

    VEE_Vector3 m_location;
    VEE_Vector3 m_velocity;
    VEE_Rgba    m_color;
  };

  Point & getDelayedPoint(uint delay) 
  { return m_points.getNewest(delay); }

  const Point & getDelayedPoint(uint delay) const 
  { return m_points.getNewest(delay); }

  // typedef std::vector<Point>::iterator iterator;


protected:

  MJ_RingBufferDelay<Point> m_points;
  
  uint  m_nofPoints;
  float m_dtSum;
  // float m_threshold;

  VEE_Vector3 m_prevLocation;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** A particle with moving tail*/

class VEE_TailParticle2 : public VEE_TailParticle
{
public:
  VEE_TailParticle2();
  virtual ~VEE_TailParticle2();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Variable getVariable(const char *) const;

  virtual void update(float);
  
  virtual VEE_Object *clone() const;
  static VEE_Object * factory();

  virtual void resizeTail(uint);

  class Link {
  public:
    float m_distance;
  };

protected:

  MJ_RingBufferDelay<Link> m_links;

  float m_springConstant;
  float m_tailForceAttenuation;

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** A particle with a moving tail*/

class VEE_TailParticle3 : public VEE_TailParticle
{
public:
  VEE_TailParticle3();
  virtual ~VEE_TailParticle3();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Variable getVariable(const char *) const;

  virtual void update(float);
  
  virtual VEE_Object *clone() const;
  static VEE_Object * factory();

protected:

  float m_velocityScale;
};

#endif
