/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_VALUE_RANDOM_H
#define VEE_VALUE_RANDOM_H

#include <vee_value.h>
#include <vee_matrix3.h>

/// Random vector producer.
class VEE_ValueRandomVector3 : public VEE_Value
{
public:
  VEE_ValueRandomVector3();
  virtual ~VEE_ValueRandomVector3();
  
  virtual VEE_Value * clone() const;
  virtual bool setValue(const char *, VEE_Value *);

  virtual bool getVector3(VEE_Vector3 *);

  void setRanges(const VEE_Vector3 &minVect, const VEE_Vector3 &maxVect)
  { m_randMin = minVect; m_randMax = maxVect; }

  void setRotation(const VEE_Matrix3 &rot) { m_randRotation = rot; }
  
protected:
  VEE_Vector3 m_randMin, m_randMax;
  VEE_Matrix3 m_randRotation;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Random rotation matrix producer.
class VEE_ValueRandomRotation : public VEE_Value
{
public:
  VEE_ValueRandomRotation();
  virtual ~VEE_ValueRandomRotation();
  
  virtual VEE_Value * clone() const;
  virtual bool setValue(const char *, VEE_Value *);
  
  virtual bool getMatrix3(VEE_Matrix3 *);
  
  void setAxes(const VEE_Vector3 &axes) { m_axes = axes; }
  void setMaxRotation(float v) { m_maxRotation = v; }
  
protected:
  VEE_Vector3 m_axes;
  float       m_maxRotation;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Random float producer
class VEE_ValueRandomFloat : public VEE_Value
{
public:
  VEE_ValueRandomFloat(float fmin = 0.0, float fmax = 1.0);
  virtual ~VEE_ValueRandomFloat();
  
  virtual VEE_Value * clone() const;
  virtual bool setValue(const char *, VEE_Value *);

  virtual bool getFloat(float *);
  virtual bool getInt(int *);

  void set(float fmin, float fmax) 
  { m_min = fmin; m_max = fmax, m_range = fmax - fmin; }
  
protected:
  float m_min, m_max, m_range;
};

#endif
