/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_values.h>

#include <vee_looks.h>
#include <vee_volume.h>

VEE_Value * VEE_ValueFloat::clone() const
{
  return new VEE_ValueFloat(*this);
}

bool VEE_ValueFloat::setValue(const char *, VEE_Value *v)
{
  float tmp;
  bool ok = v->getFloat(&tmp);
  if(ok) m_val = tmp;
  return ok;
}

bool VEE_ValueFloat::getFloat(float *v)
{
  *v = m_val;
  return true;
}

/// Get integer, rounding to the nearest value.
bool VEE_ValueFloat::getInt(int *v)
{
  *v = (m_val > 0.0) ? (int) (m_val + 0.5f) : (int) (m_val - 0.5f);
  return true;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

VEE_Value * VEE_ValueInt::clone() const
{
  return new VEE_ValueInt(*this);
}

bool VEE_ValueInt::setValue(const char *, VEE_Value *v)
{
  int tmp;
  bool ok = v->getInt(&tmp);
  if(ok) m_val = tmp;
  return ok;
}

bool VEE_ValueInt::getFloat(float *v)
{
  *v = m_val;
  return true;
}

bool VEE_ValueInt::getInt(int *v)
{
  *v = m_val;
  return true;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

VEE_Value * VEE_ValueVector3::clone() const
{
  return new VEE_ValueVector3(*this);
}

bool VEE_ValueVector3::setValue(const char *, VEE_Value *v)
{
  VEE_Vector3 tmp;
  bool ok = v->getVector3(&tmp);
  if(ok) m_val = tmp;
  return ok;
}

bool VEE_ValueVector3::getVector3(VEE_Vector3 *v)
{
  *v = m_val;
  return true;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

VEE_Value * VEE_ValueMatrix3::clone() const
{
  return new VEE_ValueMatrix3(*this);
}

bool VEE_ValueMatrix3::setValue(const char *, VEE_Value *v)
{
  VEE_Matrix3 tmp;
  bool ok = v->getMatrix3(&tmp);
  if(ok) m_val = tmp;
  return ok;
}


bool VEE_ValueMatrix3::getMatrix3(VEE_Matrix3 *v)
{
  *v = m_val;
  return true;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

VEE_Value * VEE_ValueRgba::clone() const
{
  return new VEE_ValueRgba(*this);
}

bool VEE_ValueRgba::getVector3(VEE_Vector3 *v)
{
  v->make(m_color.red(), m_color.green(), m_color.blue());
  return true;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


VEE_ValueLooks::~VEE_ValueLooks()
{}

VEE_Value * VEE_ValueLooks::clone() const
{
  return new VEE_ValueLooks(*this);
}

void VEE_ValueLooks::setLooks(VEE_Looks *l)
{
  m_looks = l;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/* 
VEE_ValueVolume::VEE_ValueVolume(const VEE_ValueVolume &l)
  : m_volume(l.m_volume ? l.m_volume->clone() : 0)
{}

VEE_ValueVolume::~VEE_ValueVolume()
{
  delete m_volume;
}

VEE_Value * VEE_ValueVolume::clone() const
{
  return new VEE_ValueVolume(*this);
}

void VEE_ValueVolume::setVolume(VEE_Volume *l)
{
  delete m_volume;
  m_volume = l;
}

VEE_ValueVolume & VEE_ValueVolume::operator = (const VEE_ValueVolume &l)
{
  delete m_volume;
  m_volume = l.m_volume ? l.m_volume->clone() : 0;

  return *this;
}
*/
