/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_VALUES_H
#define VEE_VALUES_H

#include <vee_looks.h>
#include <vee_value_ptr.h>

#include <vee_matrix3.h>

class VEE_Looks;
class VEE_Volume;

/// Value that carries one floating point number.
class VEE_ValueFloat : public VEE_Value
{
public:
  VEE_ValueFloat(float v = 0) : m_val(v) {}
  virtual ~VEE_ValueFloat() {}

  virtual VEE_Value * clone() const;
  virtual bool setValue(const char *, VEE_Value *);

  virtual bool getFloat(float *);
  virtual bool getInt(int *);

  virtual void setValue2(float v) { m_val = v; }
  float getValue() const { return m_val; }
protected:
  float m_val;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Value that carries one integer.
class VEE_ValueInt : public VEE_Value
{
public:
  VEE_ValueInt(int v = 0) : m_val(v) {}
  virtual ~VEE_ValueInt() {}

  virtual VEE_Value * clone() const;
  virtual bool setValue(const char *, VEE_Value *);

  virtual bool getFloat(float *);
  virtual bool getInt(int *);

  virtual void setValue2(int v) { m_val = v; }
  int getValue() const { return m_val; }
protected:
  int m_val;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueVector3 : public VEE_Value
{
public:
  VEE_ValueVector3(float x, float y, float z) { m_val.make(x, y, z); }
  VEE_ValueVector3(const VEE_Vector3 &v = VEE_Vector3ZERO) : m_val(v) {}
  virtual ~VEE_ValueVector3() {}

  virtual VEE_Value * clone() const;
  virtual bool setValue(const char *, VEE_Value *);

  virtual bool getVector3(VEE_Vector3 *);

  void setValue2(const VEE_Vector3 &v) { m_val = v; }
  const VEE_Vector3 & getValue() const { return m_val; }
protected:
  VEE_Vector3 m_val;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueMatrix3 : public VEE_Value
{
public:
  VEE_ValueMatrix3(const VEE_Matrix3 &v = VEE_Matrix3IDENTITY) 
    : m_val(v) {}
  virtual ~VEE_ValueMatrix3() {}

  virtual VEE_Value * clone() const;
  virtual bool setValue(const char *, VEE_Value *);

  virtual bool getMatrix3(VEE_Matrix3 *);

  void setValue2(const VEE_Matrix3 &v) { m_val = v; }
  const VEE_Matrix3 & getValue() const { return m_val; }
  
protected:
  VEE_Matrix3 m_val;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueRgba : public VEE_Value
{
public:
  VEE_ValueRgba() : m_color(VEE_Rgba::White) {}
  VEE_ValueRgba(const VEE_Rgba &color) : m_color(color) {}
  virtual ~VEE_ValueRgba() {}

  virtual VEE_Value * clone() const;
  virtual bool getVector3(VEE_Vector3 *);

  void setValue2(const VEE_Rgba &color) { m_color = color; }
  const VEE_Rgba & getValue() const { return m_color; }
  
protected:
  VEE_Rgba m_color;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueLooks : public VEE_Value
{
public:
  VEE_ValueLooks() : m_looks(0) {}
  virtual ~VEE_ValueLooks();

  virtual VEE_Value * clone() const;

  VEE_Looks *looks() { return m_looks.ptr(); }
  
  void setLooks(VEE_Looks *);

protected:
  VEE_ClonablePointer<VEE_Looks> m_looks;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

typedef VEE_ValuePtr<VEE_Volume> VEE_ValueVolume;


#endif
