/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_volume_disk.h>

#include <vee_bounding_box.h>
#include <vee_random.h>

#include <sys/types.h>
#include <math.h>

VEE_VolumeDisk::VEE_VolumeDisk()
  // m_rotation(VEE_Matrix3IDENTITY)
{
  m_rotation.identity();
  m_random = new VEE_RandomUniform;
}

VEE_VolumeDisk::~VEE_VolumeDisk()
{
  if(m_random) delete m_random;
}


VEE_Vector3 VEE_VolumeDisk::newRandomPoint()
{
  VEE_Vector3 randomVect;

  randomVect[0] = m_random->rand() * m_radius;
  randomVect[1] = 0;
  randomVect[2] = m_random->rand() * m_radius;

  return m_rotation * randomVect;
}

VEE_Volume * VEE_VolumeDisk::clone() const
{
  return new VEE_VolumeDisk(*this);
}

void VEE_VolumeDisk::getBoundingBox(VEE_BoundingBox *box)
{
  VEE_Vector3 r(m_radius, m_radius, m_radius);

  box->low() = -r;
  box->high() = r;
}
