%{
#include "ConfigReader.hpp"
#include <assert.h>
#include <stdio.h>
using namespace ConfigReader;
static Chunk* currentChunk = 0;  
static Config* currentConfig = 0;  
std::string currentVariable;
std::string currentCunkName;
// #define ConfigReader_wrap yywrap
//  using std;
%}
SYMCHAR [^\n{}=/ \\]
VALCHAR [^\n{}=/]
VALUE   [^"\n\t{}=]*[^"\t\n{}= ]
PVALUE  ["].*["]
BLANK [ \n\t]

%x comment
%x chunkstart
%x inchunk
%x variable
%x value
%option stack
%%
<*>"/*"         yy_push_state(comment);
<comment>[^*\n]*        /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   /* eat up '*'s not followed by '/'s */
<comment>\n             /* if line number were counted ... */
<comment>"*"+"/"        yy_pop_state();

<INITIAL>{BLANK}*                          /* eat spaces */
<INITIAL>{SYMCHAR}+ {
              BEGIN(chunkstart); 
	      currentCunkName = yytext;
	      currentChunk = new Chunk();
                }
<chunkstart>{BLANK}*"{" BEGIN(inchunk);
<inchunk>"}" BEGIN(INITIAL); if(currentChunk) currentConfig->set(currentCunkName, *currentChunk);
<inchunk>{BLANK}*
<inchunk>{SYMCHAR}+ {
  BEGIN(variable);
  currentVariable = yytext;
  }
<variable>{BLANK}*={BLANK}* BEGIN(value);
<value>{VALUE} { BEGIN(inchunk); if(currentChunk) currentChunk->set(currentVariable,Variant(yytext));}
<value>{PVALUE} { 
  BEGIN(inchunk); 
  assert(yytext[0] == '"' && yytext[strlen(yytext)-1] == '"');
  std::string text = yytext+1;
  text.resize(text.size()-1);
  if(currentChunk) 
    currentChunk->set(currentVariable,Variant(text)); 
}

<variable>.   std::cerr << "Parse error: Expected \"=\", but found "<<yytext<<" instead\n";
<value>.      std::cerr << "Parse error: " << yytext << " is illegal value for variable " << currentVariable << '\n';
<inchunk>.    std::cerr << "Parse error: Expected a variable name in chunk " << currentCunkName << ", but found " << yytext << " instead\n";
<chunkstart>.  std::cerr << "Parser error in chunk: " << currentCunkName << '\n';

%%

int ConfigReader_wrap (void )
{
  return 0;
}

namespace ConfigReader {

  static void (*lockFunc)(bool) = 0;

  void setLockFunction(void (*function)(bool))
  {
    puts("ConfigReader::setLockFunction");

    lockFunc = function;
  }


  bool readConfig(Config *c, const char *filename)
  {
    puts("ConfigReader::readConfig");

    if(lockFunc) (*lockFunc) (true);
    
    currentConfig = c;
    
    assert(filename);
    yyin = fopen(filename,"rb");

    if(yyin) {
      ConfigReader_lex();
      fclose(yyin);
      currentConfig = 0;
      
      if(lockFunc) (*lockFunc) (false);
      
      puts("ConfigReader::readConfig # EXIT 2");
      return true;
    }
    currentConfig = 0;
    
    if(lockFunc) (*lockFunc) (false);
    
    puts("ConfigReader::readConfig # EXIT 2");

    return false;
  }
}
