/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_solar_conversion.h>

#include <gee_group.h>
#include <gee_triangle_mesh.h>

#include <vee_error.h>

#include <Model.hpp>
#include <GraphicTriangles.hpp>

using namespace Solar;

void GEE_ToSolar::convert(Solar_Model * target, GEE_Group * source)
{
  const char * fname = "GEE_ToSolar::convert";

  uint triCount = 0;

  for(GEE_Group::iterator it = source->begin(); it != source->end(); it++) {
    VEE_RefPtr<GEE_Geometry> * refg = source->getGeometry(it);
    
    if(!refg)
      continue;

    GEE_Geometry * g = refg->ptr();

    GEE_TriangleMesh * gt = dynamic_cast<GEE_TriangleMesh *> (g);

    if(gt) {
      GraphicTriangles * st = new GraphicTriangles();

      st->setMaterial(gt->materialName());

      for(GEE_TriangleMesh::triangleIterator tit = gt->beginTriangle();
	  tit != gt->endTriangle(); tit++) {

	GEE_TriangleIndices & ti = GEE_TriangleMesh::getTriangle(tit);

	GraphicTriangles::Triangle tri;
	tri.clear();

	for(uint v = 0; v < 3; v++) {
	  GEE_PolygonVertex * vert = gt->getVertex(ti.m_indices[v]);

	  assert(vert);

	  tri.m_vertices[v].m_vertex = vert->m_location;
	  tri.m_vertices[v].m_normal = vert->m_normal;
	  tri.m_vertices[v].m_uv     = vert->m_uv;
	}

	st->addTriangle(tri);
      }

      // st->write( & std::cout);

      triCount += (int) st->nofTriangles();
      
      VEE_trace("%s # Adding triangle mesh (%s) with %d triangles", 
		fname, gt->materialName().c_str(), (int) st->nofTriangles());
      
      target->addGraphic(st);
    }
    
  }
  
  VEE_trace("%s # Model finished with %u triangles", fname, triCount);
}
