/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_mesh_conversion.h>

#include <vee_volume_grid.h>

#include <GraphicTriangles.hpp>

void VEE_MeshConversion::append(Solar::GraphicTriangles * target,
				const VEE_MarchingCubes * source,
				/// Texture coordinate scaler
				float uvScale)
{

  Solar::GraphicTriangles::Triangle solartri;

  const VEE_MarchingCubes::Triangle * veetri = & source->triangle(0);

  uint n = source->triangleCount();

  for(uint i=0; i < n; i++) {
    solartri.clear();
    VEE_MarchingCubes::Triangle t = *veetri;

    solartri.m_vertices[0].m_vertex = t.m_vertices[0];
    solartri.m_vertices[1].m_vertex = t.m_vertices[1];
    solartri.m_vertices[2].m_vertex = t.m_vertices[2];

    VEE_Vector3 n = cross(t.m_vertices[1] - t.m_vertices[0], 
			  t.m_vertices[2] - t.m_vertices[0]);
    n.normalize();
    
    solartri.setNormals(n);

    for(uint j=0; j < 3; j++) {
      solartri.m_vertices[j].m_uv.x = 
	uvScale * solartri.m_vertices[j].m_vertex.x;
      solartri.m_vertices[j].m_uv.y = 
	uvScale * solartri.m_vertices[j].m_vertex.y;
    }

    target->addTriangle(solartri);

    veetri++;
  }
}
  

void VEE_MeshConversion::append(Solar::GraphicTriangles * target,
				const VEE_VolumeGrid2   * source,
				/// Texture coordinate scaler
				float uvScale)
{
  uint n = source->gridCount();
  
  for(uint i=0; i < n; i++)
    append(target, & source->grid(i)->m_triangles, uvScale);
}
