/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_OPCODE_COLLISION_H
#define VEE_OPCODE_COLLISION_H

#include <vee_particle2.h>

#include <vee_matrix4.h>

#include <Opcode.h>
#include <OPC_RayCollider.h>

#include <vector>

class VEE_BoundingBox;

/** Strict hard-body collisions based on the OPCODE library. */

class VEE_OpcodeCollision : public VEE_Particle2
{
  friend class VEE_GlRenderer;

public:
  VEE_OpcodeCollision();
  virtual ~VEE_OpcodeCollision();
  
  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Variable getVariable(const char *) const;
  virtual void update(float);

  virtual void render(VEE_Renderer *);
  virtual VEE_Object * clone() const;

  bool loadModel(const char *file);

  void meshAddTri(VEE_Vector3 a,VEE_Vector3 b,VEE_Vector3 c);
  void meshAddQuad(VEE_Vector3 a,VEE_Vector3 b,VEE_Vector3 c,VEE_Vector3 d);
  void meshClear(void);
  bool meshCommit(void);

protected:

  bool mMeshCommitted;

  bool   m_visible;
  bool   m_active;

  void recursiveImpact(VEE_SceneBox *scene, VEE_BoundingBox *bb);

  // collision handler:
  //	called when an object collides with the collision mesh.
  //	if this returns true, the collision affects object velocity.
  //	if false, the object continues as if no collision happened.
  virtual bool onCollide(VEE_Object *o);

  struct Triangle { int m_indices[3]; };

  float       m_scale;
  VEE_Matrix3 m_rotation;
  VEE_Matrix4 m_opcMatrix;
  float       m_bounce;
  float       m_drag;

  Opcode::OPCODECREATE   m_opcCreate;
  Opcode::MeshInterface  m_meshinterface;
  Opcode::CollisionFaces m_collisionFaces;
  Opcode::RayCollider    m_rayCollider;
  Opcode::Model          m_opcModel;
  
  std::vector<VEE_Vector3> m_vertices;
  std::vector<VEE_Vector3> m_normals;
  std::vector<Triangle>    m_triangles;

  float m_dt;
};

#endif
