/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_COLOR_AROUND_PATH_H
#define GEE_COLOR_AROUND_PATH_H

#include <gee_path_to_x.h>
#include <gee_recolor.h>
#include <gee_sphere_group.h>

#include <vector>

class GEE_ColorAroundPath : public GEE_PathToX, 
			    public GEE_ReColor, 
			    public GEE_SphereGroup
{
public:
  GEE_ColorAroundPath(GEE_id_t pathId = -1, 
		      bool vertexColors = true,
		      float radiusNorm = 1.0f,
		      float exaggerateRadius = 0.0f,
		      bool useNapra = false);
  virtual ~GEE_ColorAroundPath();
  
  virtual void getBoundingBox(VEE_BoundingBox *);
  virtual bool recolor(const VEE_Vector3 &location,
		       VEE_Vector4 * color);
  virtual bool recolor(const VEE_Vector3 &location,
		       VEE_Rgba * color);
  virtual void recolored(GEE_Geometry *);

  virtual VEE_Rgba basicColor();

  virtual bool act(GEE_World *, const GEE_TimeStamp &);

  virtual bool read(GEE_Io *);
  virtual bool write(GEE_Io *);
  virtual const char * type();
  static GEE_Operator * create();

  void setColor(const VEE_Rgba c) { m_color = c; }
  void setRadius(float r) { m_radius = r; }
  void setPressure(float p) { m_pressure = p; }

protected:

  virtual bool handleNewVertex(GEE_Path3D::iterator &);

  void doColor(GEE_World *);

  VEE_Rgba             m_color;
  std::set<GEE_id_t>   m_hits;
  float                m_pressure;
  bool                 m_vertexColors;

  float    m_exaggerateRadius;
  float    m_radiusNorm;

  bool     m_useNapra;
};

#endif
