/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_COPY_GEOMETRY_H
#define GEE_COPY_GEOMETRY_H

#include <gee_operator.h>
#include <gee_world.h>

#include <vee_matrix4.h>

/** Takes a copy of some geometry. The layer of this operator is
    applied to all copies. One can also apply a transform to the
    copied object. If data is copied from gallery then the center of
    the gallery item is located at the transformation origin.

    If one defines the internal GEE_World object to have some content,
    then the geometry is copied from it rather than from some other
    source.

    @author Tommi Ilmonen
*/

class GEE_CopyGeometry : public GEE_Operator
{
public:
  GEE_CopyGeometry(GEE_id_t sourceId = -1,
		   bool sourceIsGeometry = true);
  virtual ~GEE_CopyGeometry();

  virtual id_container * getExtraIds();
  virtual bool remapIdMembers(id_remap &);

  virtual bool act(GEE_World *, const GEE_TimeStamp &);
  
  virtual bool read(GEE_Io *);
  virtual bool write(GEE_Io *);
  virtual const char * type();

  static GEE_Operator * create();

  void setTransformation(const VEE_Matrix4d &t) { m_transform = t; }

  GEE_World & geometries() { return m_sourceModels; }

protected:

  void updateIds();

  GEE_id_t m_sourceId;
  /** The source to be copied can be either a geometry or a gallery
      item. */
  bool     m_sourceIsGeometry;
  VEE_Matrix4d m_transform;
  
  id_remap m_remap;
  id_container m_ids;

  /** If the world below contains geometries, then we insert data from
      it rather than copy some source. */

  GEE_World m_sourceModels;
};


#endif
