/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_ENGINE_H
#define GEE_ENGINE_H

#include <gee_collector_renderer.h>
#include <gee_gallery.h>
#include <gee_operator.h>
#include <gee_transparency_list.h>
#include <gee_world.h>

/** An engine that runs an action manager in harmony with a world. */

class GEE_Engine
{
public:
  GEE_Engine();
  ~GEE_Engine();

  void addOperator(GEE_Operator *);

  void update(GEE_TimeStamp);
  void rewind(GEE_TimeStamp);
  void toBegin();

  void undo(bool update = true, uint n = 1);
  void redo(bool update = true, uint n = 1);

  bool read(const char *, bool merge = false, bool clean = true);
  bool write(const char *);
  bool mergeGalleriesFrom(const char *);

  GEE_World           & world()   { return m_world; }
  GEE_OperatorManager & actions() { return m_actions; }
  GEE_Gallery         & gallery() { return m_gallery; }

  void setGarbageBin(VEE_GarbageBin * bin);

  void collectObjects(bool clearOld = true);
  void clear();

  GEE_CollectorRenderer & collector() { return m_collector; }

protected:

  static void remapThese(GEE_OperatorManager &, GEE_World &, GEE_Gallery &);

  GEE_Gallery           m_gallery;
  GEE_World             m_world;
  GEE_OperatorManager   m_actions;
  GEE_TimeStamp         m_prevTime;
  GEE_CollectorRenderer m_collector;

  static bool m_debug;
};

#endif
