/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_PARTICLE_ROTATE_H
#define GEE_PARTICLE_ROTATE_H

#include <gee_param_writer.h>
#include <gee_path_to_x.h>

#include <vector>

/** An operator to adjust the alignment (rotation) of particle
    billboards. 

    @author Tommi Ilmonen.
*/

class GEE_ParticleRotate : public GEE_PathToX
{
public:
  GEE_ParticleRotate(GEE_id_t pathId = -1);
  virtual ~GEE_ParticleRotate();
  
  virtual bool act(GEE_World *, const GEE_TimeStamp &);

  virtual bool read(GEE_Io *);
  virtual bool write(GEE_Io *);
  virtual const char * type();

  static GEE_Operator * create();

  void setPressure(float pressure) { m_pressure = pressure; }
  void setOmega(float omega) { m_omega = omega; }

  void setRadius(float r)      { m_radius = r; }
  void setRadiusBoost(float r) { m_radiusBoost = r; }
  void setRadiusNorm(float r)  { m_radiusNorm = r; }
  void setUseNapra(bool use)   { m_useNapra = use; }

protected:
  
  void doTurn(GEE_World *);

  virtual bool handleNewVertex(GEE_Path3D::iterator &);

  GEE_ParamWriter   m_writer;

  float             m_pressure;
  float             m_omega;
  float             m_radius;
  float             m_radiusBoost;
  float             m_radiusNorm;
  bool              m_useNapra;

  GEE_PathVertex3D  m_last;

  class Item
  {
  public:
    Item() {}
    ~Item() {}

    VEE_Vector3 m_location;
    float       m_radius;
    float       m_turn;
  };

  std::vector<Item> m_turns;
};

#endif

