/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_path_insert.h>

#include <gee_io.h>
#include <gee_world.h>

#include <vee_error.h>

GEE_PathInsert::GEE_PathInsert(GEE_id_t id)
  : m_pathId(id)
{
  addIdMember(&m_pathId);
}

GEE_PathInsert::~GEE_PathInsert()
{}

/** We do not check for time-stamp. It is assumed that the caller
    knows when this method can be called. */

bool GEE_PathInsert::act(GEE_World *w, const GEE_TimeStamp &)
{
  // VEE_trace("GEE_PathInsert::act");

  if(m_pathId < 0)
    m_pathId = GEE_IdContainer::allocateId();

  else {
    GEE_Path3D * oldPath = dynamic_cast<GEE_Path3D *> (w->geometry(m_pathId));
    if(oldPath) {
      if(*oldPath == m_path) {
	oldPath->copyBasics(m_path);
	// VEE_trace("GEE_PathInsert::act # EXIT 1");
	return true;
      }
    }
  }
  
  m_path.setId(m_pathId);
  m_path.setLayer(m_layer);
  w->addGeometry(m_path.clone());

  // VEE_trace("GEE_PathInsert::act # EXIT 2");

  return true;
}

bool GEE_PathInsert::read(GEE_Io * io)
{
  std::string str;
  io->readNullTerminatedStdString(str);
  if(str != type())
    return false;
  
  readBasics(io);

  int v = io->readInt32();

  if(v != 1)
    return false;

  m_pathId = io->readInt64();

  return m_path.read(io);
}

bool GEE_PathInsert::write(GEE_Io * io)
{
  io->writeNullTerminatedString(type());
  writeBasics(io);

  io->writeInt32(1); // version
  io->writeInt64(m_pathId);

  m_path.write(io);

  return true;  
}


GEE_Operator * GEE_PathInsert::create()
{
  return new GEE_PathInsert;
}
