/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_path_to_camera.h>

#include <gee_io.h>
#include <gee_macros.h>
#include <gee_world.h>

#include <vee_math.h>

GEE_PathToCamera::GEE_PathToCamera(GEE_id_t pathId, 
				   GEE_id_t cameraId)
  : GEE_PathToX(pathId),
    m_cameraId(cameraId),
    m_fov(70.0f),
    m_world(0)
{
  addIdMember(&m_cameraId);

  m_writer.addMemberFloat(GEE_CONST2('F', 'O'), & m_fov);
  m_writer.addMemberMatrix3(GEE_CONST2('C', 'R'), & m_foo);

  /* VEE_Matrix3 r1, r2;

  r1.rotateY(VEE_HALF_PI);
  r2.rotateZ(VEE_HALF_PI); */

  // m_cameraRot = r1 * r2;
  // m_cameraRot = r1;
  m_cameraRot.identity();
  // m_cameraRot.rotateY(0.0);
}

GEE_PathToCamera::~GEE_PathToCamera()
{}

bool GEE_PathToCamera::start(GEE_World * w, const GEE_TimeStamp & now)
{
  m_world = w;

  return GEE_PathToX::start(w, now);
}

bool GEE_PathToCamera::read(GEE_Io * io)
{
  std::string str;
  io->readNullTerminatedStdString(str);
  if(str != type())
    return false;
  
  readBasics(io);
  readPathId(io);


  int v = io->readInt32();

  if(v != 1)
    return false;

  return m_writer.readParameters(io);
}
 
bool GEE_PathToCamera::write(GEE_Io * io)
{
  io->writeNullTerminatedString(type());

  writeBasics(io);
  writePathId(io);

  io->writeInt32(1); // file-format version 

  return m_writer.writeParameters(io);
}
 
const char * GEE_PathToCamera::type() 
{ 
  return "GEE_PathToCamera"; 
}

GEE_Operator * GEE_PathToCamera::create()
{
  return new GEE_PathToCamera();
}
  
bool GEE_PathToCamera::handleNewVertex(GEE_Path3D::iterator & it)
{
  if(m_cameraId < 0)
    m_cameraId = allocateId();
  
  const GEE_PathVertex3D &pv  = GEE_Path3D::getVertex(it);

  GEE_CameraParameters cam;  

  VEE_Matrix3 rot = pv.m_rotation * m_cameraRot;
  // VEE_Matrix3 rot = m_cameraRot * pv.m_rotation;

  cam.m_location  = pv.m_location;
  cam.m_direction = rot * VEE_Vector3(0.1, -1, 0.1);
  cam.m_up        = rot * VEE_Vector3(1, 0, 0);
  cam.m_fov       = m_fov;

  m_world->setCamera(m_cameraId, cam);

  return true;
}
