/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_PATH_TO_LINES_H
#define GEE_PATH_TO_LINES_H

#include <gee_lines.h>
#include <gee_param_writer.h>
#include <gee_path_to_x.h>

class GEE_PathToLines : public GEE_PathToX
{
public:
  GEE_PathToLines(GEE_id_t pathId = -1, 
		  GEE_id_t lineId = -1);
  virtual ~GEE_PathToLines();

  virtual bool start(GEE_World *, const GEE_TimeStamp &);
  virtual bool stop(GEE_World *, const GEE_TimeStamp &);

  virtual bool read(GEE_Io *);
  virtual bool write(GEE_Io *);
  virtual const char * type() { return "GEE_PathToLines"; }
  static GEE_Operator * create();

  GEE_Material & material() { return m_material; }
  void setMaterial(const GEE_Material &m) { m_material = m; }

  void setWidth(float w) { m_lineWidth = w; }

  void setUseNapra(bool use) { m_useNapra = use; }
  void setNapraRadius(float radius) { m_napraRadius = radius; }

protected:

  virtual bool handleNewVertex(GEE_Path3D::iterator &);

  GEE_ParamWriter m_writer;

  GEE_id_t        m_lineId;

  GEE_Material    m_material;
  float           m_lineWidth;
  float           m_napraRadius;

  bool            m_useNapra;

  VEE_RefPtr<GEE_Geometry> m_lines;
};

#endif
