/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_PATH_TO_MESH_H
#define GEE_PATH_TO_MESH_H

#include <gee_path_to_x.h>
#include <gee_profile.h>

#include <vee_reference_object.h>

class GEE_TriangleMesh;

/** Converts paths to a triangle mesh with the aid of a profile. */

class GEE_PathToMesh : public GEE_PathToX
{
public:
  GEE_PathToMesh(GEE_id_t pathId = -1, 
		 GEE_id_t meshId = -1);
  virtual ~GEE_PathToMesh();

  virtual bool start(GEE_World *, const GEE_TimeStamp &);
  virtual bool stop(GEE_World *, const GEE_TimeStamp &);

  virtual bool read(GEE_Io *);
  virtual bool write(GEE_Io *);
  virtual const char * type() { return "GEE_PathToMesh"; }

  const GEE_Profile &profile() const { return m_profile; }
  void setProfile(const GEE_Profile &profile) 
  { m_profile = profile; }

  static void normalizeProfile(GEE_Profile * profile);

  GEE_TriangleMesh * triMesh();

  static GEE_Operator * create();

  GEE_Material & material() { return m_material; }
  void setMaterial(const GEE_Material &m) { m_material = m; }

  void setScale(float scale) { m_scale = scale; }
  void setRadius(float radius) { m_radius = radius; }
  void setThreshold(float t) { m_threshold = t; }

  void setAlignment(GEE_AlignMode m) { m_align = m; }

  void useNapra(bool withNapra) 
  { m_withNapra = withNapra; m_checkNapra = false; }

  void setSpecialFinger(int f)
  { m_specialFinger = f; }

  static VEE_Matrix4 createMatrix(const GEE_PathVertex3D &,
				  float scale);

  float scale() const { return m_scale; }
protected:

  virtual bool handleNewVertex(GEE_Path3D::iterator &);

  void setVerticesWithoutNapra(GEE_PathVertex3D & tv, 
			       GEE_TriangleMesh * mesh);
  void setVerticesWithNapra(GEE_PathVertex3D & tv, 
			    GEE_TriangleMesh * mesh);

  GEE_id_t    m_meshId;
  VEE_Vector3 m_prevLoc;
  VEE_Vector3 m_dir;

  GEE_Material         m_material;
  float                m_scale;
  float                m_radius;
  float                m_threshold;
  int                  m_align;
  bool                 m_newSegment;
  int                  m_segmentCount;
  bool                 m_withNapra;
  bool                 m_checkNapra;
  int                  m_specialFinger;

  VEE_RefPtr<GEE_Geometry> m_mesh;

  GEE_Profile m_profile;
};

#endif
