/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_PATH_TO_X_H
#define GEE_PATH_TO_X_H

#include <gee_edit_set.h>
#include <gee_operator.h>
#include <gee_path.h>

class GEE_PathToX : public GEE_Operator
{
public:
  GEE_PathToX(GEE_id_t pathId = -1, 
	      GEE_id_t pathId2 = -1);
  virtual ~GEE_PathToX();

  virtual bool start(GEE_World *, const GEE_TimeStamp &);
  virtual bool act(GEE_World *, const GEE_TimeStamp &);
  virtual bool stop(GEE_World *, const GEE_TimeStamp &);

  bool readPathId(GEE_Io *);
  bool writePathId(GEE_Io *);

  uint processedPathVertices() const { return m_processedPathVertices; }

  void setShowToolSphere(bool show) { m_showToolSphere = show; }
  void setToolSphereRadius(float r) { m_sphereRadius = r; }

  GEE_id_t pathId() const { return m_pathId; }

protected:

  virtual bool handleNewVertex(GEE_Path3D::iterator &);
  virtual bool handleNewVertices(GEE_Path3D::iterator &, 
				 GEE_Path3D::iterator &);

  void addEditedGeometry(GEE_Geometry *g) { m_edits.addGeo(g); }
private:

  bool initializePath(GEE_World *w, GEE_id_t id);

  bool handlePath(GEE_World *, const GEE_TimeStamp & tworld);
  bool handleTwoPaths(GEE_World *, const GEE_TimeStamp & tworld);

  inline GEE_Path3D * getPath(GEE_World *, GEE_id_t);

  GEE_id_t             m_pathId;
  GEE_id_t             m_pathId2;
  GEE_id_t             m_sphereId;

  int                  m_pathIterator1;
  int                  m_pathIterator2;
  bool                 m_nodeIsUsed;
  bool                 m_showToolSphere;
  float                m_sphereRadius;
  GEE_EditSet          m_edits;

  
  uint m_processedPathVertices;
};

#endif
