/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_POLYGON_MESH_H
#define GEE_POLYGON_MESH_H

#include <gee_triangle_mesh.h>

/** Stores indices to polygon vertex points. Each polygon is
    represented by a list of integers that correspond to some
    vertices. */

class GEE_PolygonIndices
{
public:

  GEE_PolygonIndices() {}

  void read(GEE_Io *, int);
  void write(GEE_Io *, int);
  
  void addVertex(int v) { m_indices.push_back(v); }

  std::vector<int> m_indices;
};

/** A polygon mesh. Works pretty much like a triangle mesh
    (GEE_TriangleMesh). 

    @author Tommi Ilmonen
*/

class GEE_PolygonMesh : public GEE_Geometry
{
public: 
  typedef std::map<int, GEE_PolygonVertex>::iterator  vertexIterator;
  typedef std::map<int, GEE_PolygonIndices>::iterator polygonIterator;

  GEE_PolygonMesh();
  ~GEE_PolygonMesh();

  virtual GEE_Geometry * clone() const;
  virtual void render(GEE_Renderer *);
  virtual bool getBoundingBox(VEE_BoundingBox *);
  virtual bool move(const VEE_Vector3 & offset);


  void clear();
  int addVertex(const GEE_PolygonVertex &);
  int addPolygon(const GEE_PolygonIndices &);

  GEE_PolygonVertex * getVertex(int);

  polygonIterator beginPolygon() { return m_polygons.begin(); }
  polygonIterator endPolygon()   { return m_polygons.end(); }

  uint polygonCount() const { return m_polygons.size(); }

  vertexIterator beginVertex() { return m_vertices.begin(); }
  vertexIterator endVertex()   { return m_vertices.end(); }

  uint vertexCount() const { return m_vertices.size(); }

  static GEE_PolygonIndices & getPolygon(polygonIterator &it)
  { return (*it).second; }

  GEE_TriangleMesh * triangleMesh();

  static GEE_PolygonMesh * cast(GEE_Geometry *d) 
  { return dynamic_cast<GEE_PolygonMesh *> (d); }

protected:

  std::map<int, GEE_PolygonVertex>   m_vertices;
  std::map<int, GEE_PolygonIndices>  m_polygons;

  int m_vertexCounter;
  int m_polygonCounter;

  VEE_BoundingBox m_bbox;
};


#endif
