/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_PROFILE_H
#define GEE_PROFILE_H

#include <gee_geometry.h>

#include <vee_vector2.h>

#include <vector>

class GEE_ProfileVertex2D
{
public:
  GEE_ProfileVertex2D(const VEE_Vector2 & location = VEE_Vector2ZERO,
		      const VEE_Vector2 & normal   = VEE_Vector2ZERO,
		      bool continues = true)
    : m_location(location), m_normal(normal), m_continues(continues)
  {}

  bool read(GEE_Io * );
  bool write(GEE_Io *);

  VEE_Vector2 m_location;
  VEE_Vector2 m_normal;
  /** Connectivity information. If this is true (=default), then the
      profile continues from this vertex to the next. Otherwise the
      profile is cut open after this vertex. */
  bool        m_continues;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Profile to be used when creating 3D tubes out of path information. 
    */

class GEE_Profile : public GEE_Geometry
{
public:
  typedef std::vector<GEE_ProfileVertex2D> container;
  typedef container::iterator iterator;

  GEE_Profile();
  virtual ~GEE_Profile();

  virtual void render(GEE_Renderer *);
  virtual bool getBoundingBox(VEE_BoundingBox *);

  virtual bool read(GEE_Io * );
  virtual bool write(GEE_Io *);

  void clear();

  void generateCircle(float radius, int segments);
  void generateBar(float height = 1.0, float topWidth = 0.0);
  void generateCross(float radius);
  
  void addVertex(const GEE_ProfileVertex2D &v) { m_vertices.push_back(v); }

  void normalize(const VEE_Vector3 & span);

  uint size() const { return m_vertices.size(); }
  void resize(uint n) { m_vertices.resize(n); }
  uint segments() const;
  GEE_ProfileVertex2D & getVertex(uint i) { return m_vertices[i]; }

  iterator begin() { return m_vertices.begin(); }
  iterator end()   { return m_vertices.end(); }

protected:

  container m_vertices;

};

#endif
