/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_renderer.h>

#include <gee_geometry.h>
#include <gee_transparency_list.h>

#include <vee_error.h>

#include <typeinfo>

static bool m_debug = false;

GEE_Renderer::GEE_Renderer()
{}

GEE_Renderer::~GEE_Renderer()
{}

/** Renders transparencies back-to-front*/

void GEE_Renderer::renderBackToFront(GEE_TransparencyList *tl)
{
  // VEE_trace("GEE_Renderer::renderBackToFront");

  GEE_TransparencyList::iterator it = tl->end();
  GEE_TransparencyList::iterator sentinel = tl->begin();

  if(it == sentinel)
    return;

  do {
    it--;

    GEE_TransparencyList::Item & i = GEE_TransparencyList::getItem(it);
    if(i.type() == GEE_TransparencyList::VEE_OBJ)
      renderVeeObject((VEE_Object *) i.ptr(), & i.transformation(), i.scale());
    else if(i.type() == GEE_TransparencyList::GEE_DATA) {
      pushTransform(i.transformation());
      enableTransforms(false);
      ((GEE_Geometry *) i.ptr())->render(this);
      enableTransforms(true);
      popTransform();
    }
    else
      VEE_error("GEE_Renderer::renderBackToFront # Unknown type %d",
		(int) i.type());

  } while(it != sentinel);
}
/** Renders objects front-to-back*/

void GEE_Renderer::renderFrontToBack(GEE_TransparencyList *tl)
{
  const char * fname = "GEE_Renderer::renderFrontToBack";

  GEE_TransparencyList::iterator sentinel = tl->end();

  for(GEE_TransparencyList::iterator it = tl->begin();
      it != sentinel; it++) {

    GEE_TransparencyList::Item & i = GEE_TransparencyList::getItem(it);

    if(i.type() == GEE_TransparencyList::VEE_OBJ)
      renderVeeObject((VEE_Object *) i.ptr(), & i.transformation(), i.scale());
    else if(i.type() == GEE_TransparencyList::GEE_DATA) {
      GEE_Geometry * d = (GEE_Geometry *) i.ptr();
      
      if(m_debug)
	VEE_trace("%s # Rendering %s", fname, typeid(*d).name());

      pushTransform(i.transformation());
      enableTransforms(false);
      d->render(this);
      enableTransforms(true);
      popTransform();
    }
    else
      VEE_error("%s # Unknown type %d", fname, (int) i.type());
  }
}
