/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_RENDERER_H
#define GEE_RENDERER_H

#include <vee_matrix4.h>

class GEE_Group;
class GEE_Lines3D;
class GEE_ParticleObject;
class GEE_Path3D;
class GEE_PolygonMesh;
class GEE_Profile;
class GEE_Sphere;
class GEE_TransparencyList;
class GEE_TriangleMesh;

class VEE_Object;

/** Abstract base class for renderers. 

    @author Tommi Ilmonen
*/

class GEE_Renderer
{
public:
  GEE_Renderer();
  virtual ~GEE_Renderer();

  virtual void renderGroupStructure(GEE_Group *) = 0;
  virtual void renderLines3D(GEE_Lines3D *) = 0;
  virtual void renderPath3D(GEE_Path3D *) = 0;
  virtual void renderParticleObject(GEE_ParticleObject *) = 0;
  virtual void renderPolygonMesh(GEE_PolygonMesh *) = 0;
  virtual void renderProfile(GEE_Profile *) = 0;
  virtual void renderSphere(GEE_Sphere *) = 0;
  virtual void renderTriangleMesh(GEE_TriangleMesh *) = 0;
  virtual void renderVeeObject(VEE_Object *, 
			       const VEE_Matrix4 * m = 0, 
			       float scale = 0.0f) = 0;

  virtual void pushTransform(const VEE_Matrix4d &) = 0;
  virtual void pushTransform(const VEE_Matrix4 &) = 0;
  virtual void popTransform() = 0;
  virtual void enableTransforms(bool enable) = 0;
  virtual bool enableTransforms() = 0;

  virtual void renderBackToFront(GEE_TransparencyList *);
  virtual void renderFrontToBack(GEE_TransparencyList *);

};

#endif
