/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_SCALE_AROUND_PATHS_H
#define GEE_SCALE_AROUND_PATHS_H

#include <gee_path_to_x.h>

#include <vector>

class GEE_ScaleAroundPaths : public GEE_PathToX
{
public:
  GEE_ScaleAroundPaths(GEE_id_t pathId1 = -1, 
		       GEE_id_t pathId2 = -1, 
		       GEE_id_t geoId = -1, 
		       bool uniform = true);
  virtual ~GEE_ScaleAroundPaths();
  
  virtual id_container * getExtraIds();

  virtual bool act(GEE_World *, const GEE_TimeStamp &);

  virtual bool read(GEE_Io *);
  virtual bool write(GEE_Io *);
  virtual const char * type();
  static GEE_Operator * create();

  void addGeometryId(GEE_id_t geoId) { m_geoIds.insert(geoId); }

protected:

  virtual bool handleNewVertices(GEE_Path3D::iterator &, 
				 GEE_Path3D::iterator &);

  virtual void doScale(GEE_World *);

  class Scale
  {
  public:
    // Locations of the hands:
    VEE_Vector3 m_move, m_center, m_size, m_prevSize;
  };

  VEE_Vector3          m_curLoc;
  VEE_Vector3          m_curSize;
  VEE_Vector3          m_grabDirection;

  std::vector<Scale>    m_scales;

  /** The id of the geometries to be scaled. */
  id_container m_geoIds;
  bool     m_uniform;
};


#endif
