/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_SPHERE_GROUP_H
#define GEE_SPHERE_GROUP_H

#include <gee_containment.h>

#include <vector>

class GEE_SphereGroup : public GEE_Containment
{
public:
  class Sphere
  {
  public:
    VEE_Vector3 m_location;
    float       m_radius;
    float       m_radiusSquared;
  };

  GEE_SphereGroup();
  ~GEE_SphereGroup();

  virtual void getBoundingBox(VEE_BoundingBox *);
  virtual bool contains(const VEE_Vector3 &);
  virtual bool intersects(const GEE_SimpleTriangle &);

  void setRadius(float r) { m_radius = r; }

  void addSphere(const VEE_Vector3 &, float radius = -1.0f);
  void clear() { m_locations.clear(); }

  const Sphere * locationBegin() const { return & * m_locations.begin(); }
  const Sphere * locationEnd()   const { return & * m_locations.end(); }

protected:
  uint                 locationCount() const { return m_locations.size(); }

  float                m_radius;

private:


  std::vector<Sphere> m_locations;
  VEE_BoundingBox     m_bbox;
};

#endif
