/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_texture_around_path.h>

#include <gee_io.h>
#include <gee_macros.h>
#include <gee_particle_object.h>
#include <gee_world.h>

#include <vee_error.h>

GEE_TextureAroundPath::GEE_TextureAroundPath(GEE_id_t pathId,
					     float radiusNorm,
					     float exaggerateRadius,
					     bool useNapra)
  : GEE_EraseAroundPath(pathId, false, radiusNorm, exaggerateRadius, useNapra),
    m_texId(-1)
{}

GEE_TextureAroundPath::~GEE_TextureAroundPath()
{}

static const ushort GEE_TextureId = GEE_CONST2('T', 'X');
static const ushort GEE_EndId     = GEE_CONST2('E', 'N');

bool GEE_TextureAroundPath::read(GEE_Io *io)
{
  if(!GEE_EraseAroundPath::read(io))
    return false;

  ushort id = io->readInt16();

  while(id != GEE_EndId) {
    if(id == GEE_TextureId)
      io->readNullTerminatedStdString(m_texName);
    else {
      VEE_error("GEE_TextureAroundPath::read # Bad id at %d (%c%c)", 
		(int) io->position(), (uchar) (id >> 8), (uchar) id);
      return false;
    }
    id = io->readInt16();
  }

  VEE_trace("GEE_TextureAroundPath::read # r = %f texName = \"%s\"", m_radius,
	    m_texName.c_str());

  return true;
}

bool GEE_TextureAroundPath::write(GEE_Io *io)
{
  GEE_EraseAroundPath::write(io);

  io->writeInt16(GEE_TextureId);
  io->writeNullTerminatedString(m_texName.c_str());
  
  io->writeInt16(GEE_EndId);

  return true;
}

const char * GEE_TextureAroundPath::type()
{
  return "GEE_TextureAroundPath";
}

GEE_Operator * GEE_TextureAroundPath::create()
{
  return new GEE_TextureAroundPath;
}

void GEE_TextureAroundPath::doWork(GEE_World *w)
{
  GEE_Geometry * g;
  // VEE_trace("GEE_TextureAroundPath::doWork");

  if(m_texId < 0) {
    m_texId = GEE_World::textureId(m_texName);
    if(m_texId < 0)
      VEE_error("GEE_TextureAroundPath::doWork # texId < 0");
  }

  for(GEE_LayerGeometryIterator it(w, m_layer); (g = *it); it.nextVisible()) {
    GEE_ParticleObject * po = dynamic_cast<GEE_ParticleObject *> (g);
    if(po) {
      po->retexture(this, m_texName, m_texId);
      // VEE_trace("GEE_TextureAroundPath::doWork # on %p (%d)", po, m_texId);
    }
  }
}
