/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_TRANSFORMATION_H
#define GEE_TRANSFORMATION_H

#include <gee_geometry.h>

#include <vee_clonable_pointer.h>
#include <vee_matrix4.h>

/** A geometrical transformation defined by 4x4 matrix. 

@author Tommi Ilmonen

*/

class GEE_Transformation : public GEE_Geometry
{
public:
  GEE_Transformation();
  virtual ~GEE_Transformation();

  virtual GEE_Geometry * clone() const;
  virtual GEE_Geometry * removeData(const VEE_Vector3 &loc, float radius);
  virtual void render(GEE_Renderer *);
  virtual bool getBoundingBox(VEE_BoundingBox *);
  virtual bool move(const VEE_Vector3 & offset);

  void setTransformation(const VEE_Matrix4 &t) { m_transformation = t; }

  VEE_Matrix4 &       transformation() { return m_transformation; }
  const VEE_Matrix4 & transformation() const { return m_transformation; }

  void setChild(GEE_Geometry *data) { m_child = data; }

protected:

  VEE_Matrix4 m_transformation;

  VEE_ClonablePointer<GEE_Geometry> m_child;
};

#endif
