/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_transparency_list.h>

#include <gee_geometry.h>

#include <vee_bounding_box.h>
#include <vee_object.h>
#include <vee_scene.h>

GEE_TransparencyList::GEE_TransparencyList()
{
  m_camPos.clear();
}

GEE_TransparencyList::~GEE_TransparencyList()
{}

/** Adds a single VEE_Object pointer to the list*/
void GEE_TransparencyList::addVee(VEE_Object *obj, const VEE_Matrix4 &m, 
				  float scale)
{
  float d = (m * obj->location() - m_camPos).length();

  if(!m_items.size())
    m_latest = m_items.insert(std::pair<float, Item>(d, Item(obj, m, scale)));
  else
    m_latest = m_items.insert(m_latest, 
			      std::pair<float, Item>(d, Item(obj, m, scale)));
}

/** Adds recursively all objects found in this scene box and boxes
    below it to the list. */
void GEE_TransparencyList::addVeeScene(VEE_SceneBox *scene, 
				       const VEE_Matrix4 &m)
{
  for(VEE_SceneBox::iterator it = scene->begin(); it != scene->end(); it++)
    addVee(VEE_SceneBox::getObject(it), m);

  if(!scene->children())
    return;

  for(uint i=0; i < VEE_SB_CHILDREN; i++)
    addVeeScene(scene->child(i), m);
}

/** Adds a GEE_Geometry pointer to the list. */

void GEE_TransparencyList::addGee(GEE_Geometry *data, const VEE_Matrix4 &m)
{
  VEE_BoundingBox bb;
  data->getBoundingBox(&bb);

  float d = (m * bb.center() - m_camPos).length();

  m_items.insert(std::pair<float, Item>(d, Item(data, m)));  

  if(m_items.size() == 1)
    m_latest = m_items.begin();
}

/** Removes all pointers. */
void GEE_TransparencyList::clear()
{
  m_items.clear();
}
