/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_ungroup_geos.h>

#include <gee_group.h>
#include <gee_io.h>
#include <gee_macros.h>
#include <gee_world.h>

#include <vee_error.h>

GEE_UnGroupGeos::GEE_UnGroupGeos(GEE_id_t groupId)
  : m_groupId(groupId)
{
  addIdMember(&m_groupId);

  m_writer.addMemberId(GEE_CONST2('G', 'R'), &m_groupId);
}

GEE_UnGroupGeos::~GEE_UnGroupGeos()
{}

bool GEE_UnGroupGeos::act(GEE_World * w, const GEE_TimeStamp &)
{
  if(m_groupId < 0)
    return false;
  
  GEE_Group * group = dynamic_cast<GEE_Group *> (w->geometry(m_groupId));

  if(!group)
    return false;

  group->breakGroup();
  
  w->eraseGeometry(m_groupId);

  return true;
}

bool GEE_UnGroupGeos::read(GEE_Io *io)
{
  std::string str;
  io->readNullTerminatedStdString(str);
  if(str != type())
    return false;
  
  readBasics(io);

  int v = io->readInt32();

  if(v != 1)
    return false;

  m_writer.readParameters(io);

  return true;
}

bool GEE_UnGroupGeos::write(GEE_Io *io)
{
  io->writeNullTerminatedString(type());

  writeBasics(io);

  io->writeInt32(1); // version

  m_writer.writeParameters(io);

  return true;
}

const char * GEE_UnGroupGeos::type()
{
  return "GEE_UnGroupGeos";
}

GEE_Operator * GEE_UnGroupGeos::create()
{
  return new GEE_UnGroupGeos();
}
