/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_gl_light.h>

#include <ConfigReader.hpp>


/** Initialiaze light values with reasonable defaults. */
  
GEE_GlLight::GEE_GlLight()
  : m_location(1.0, 1.0, 1.0, 0.0), // Last zero -> directional light
    m_ambient(0.3, 0.3, 0.3, 1.0),
    m_diffuse(1.0, 1.0, 1.0, 1.0),
    m_specular(0.4, 0.4, 0.4, 1.0)
{}

/** Apply the this these light parameters to specified OpenGL
      light. */
void GEE_GlLight::apply(GLenum id)
{
  glLightfv (id, GL_AMBIENT,  m_ambient.data());
  glLightfv (id, GL_DIFFUSE,  m_diffuse.data());
  glLightfv (id, GL_SPECULAR, m_specular.data());
  glLightfv (id, GL_POSITION, m_location.data());
}

/** Get all light parameters from a config chunk. If the values
    cannot be read, then old values are implicitly kept. */
void GEE_GlLight::set(ConfigReader::Chunk *values)
{
  values->get("Ambient").getFloats(m_ambient.data(), 4);
  values->get("Diffuse").getFloats(m_diffuse.data(), 4);
  values->get("Specular").getFloats(m_specular.data(), 4);
  values->get("Location").getFloats(m_location.data(), 4);
}
  
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

GEE_GlLights::GEE_GlLights()
{
  m_sceneAmbient.make(0.2, 0.2, 0.2, 1.0);
}

GEE_GlLights::~GEE_GlLights()
{}

void GEE_GlLights::set(ConfigReader::Config *config)
{
  clear();

  char buf[32];

  for(int i=0; i < 100; i++) {
    sprintf(buf, "Light-%d", i);

    ConfigReader::Chunk chunk = config->get(buf);
    if(chunk.size())
      m_lights[(GLenum) i + GL_LIGHT0].set(&chunk);
  }

  ConfigReader::Chunk chunk = config->get("Light-Ambient");
  chunk.get("Ambient").getFloats(m_sceneAmbient.data(), 4);
}

void GEE_GlLights::clear()
{
  m_lights.clear();
}

void GEE_GlLights::apply(bool on, bool reset)
{
  if(on)
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, m_sceneAmbient.data());

  // for(iterator it = m_lights.begin(); it != m_lights.end(); it++) {

  for(int i=0; i < 8; i++) {

    GLenum id = GL_LIGHT0 + i;

    iterator it = m_lights.find(id);

    if(it == m_lights.end()) {
      glDisable(id);
      continue;
    }
    
    if(on) {
      glEnable(id);
      if(reset)
	(*it).second.apply(id);
    }
    else {
      glDisable(id);      
    }
  }
}

void GEE_GlLights::set(GLenum lightId, const GEE_GlLight & light)
{
  m_lights[lightId] = light;
}
