/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef _VEE_GL_MATERIAL_H_
#define _VEE_GL_MATERIAL_H_

#ifdef __APPLE__
# include <OpenGL/gl.h>
#else
# include <GL/gl.h>
#endif


#include <map>
// #include <string>

class VEE_GlMaterial
{
public:

  enum Flag {
    BLEND = 0x1
  };

  VEE_GlMaterial(GLuint texId = (GLuint) -1, int flags = BLEND) 
    : m_textureId(texId), m_flags(flags) {}

  ~VEE_GlMaterial() {}

  bool hasTexture() const { return m_textureId != (GLuint) -1; }

  GLuint textureId() const { return m_textureId; }

  void useTexture() const
  { 
    if(m_textureId == (GLuint) -1) 
      glDisable(GL_TEXTURE_2D);
    else {
      glBindTexture(GL_TEXTURE_2D, m_textureId);
      glEnable(GL_TEXTURE_2D);
    }
  }

  int flags() const { return m_flags; }

protected:
  GLuint   m_textureId;
  int      m_flags;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** A collection of materials. */
class VEE_GlMaterialBank
{
public:
  VEE_GlMaterialBank() {}
  ~VEE_GlMaterialBank() {}
  
  void addMaterial(int, const VEE_GlMaterial &);

  const VEE_GlMaterial & getMaterial(int);

  static void setDefaultMaterial(const VEE_GlMaterial &m)
  { m_default = m; }

protected:
  
  static VEE_GlMaterial m_default;

  std::map<int, VEE_GlMaterial> m_materials;
};

#endif
