# Simple fire effect

# Usual initializations:
from vee_init import *
from vee import *

qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

agelimit = 2.0
gravity = 5.0

#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_Particle()
p.thisown = 0

p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(0.7, 0.0, 0.0, 1.0)
rgbatable.appendColor(color)
color.set(0.7, 0.7, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.0, 0.0, 0.0, 1.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)

#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

v1.make(-2.0, 0.0, -2.0)
v2.make(2.0, 0.5, 2.0)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)

#############################################################################

# vortex = VEE_ObjectFactory_create("vortex")
# scene.addNewObject(vortex)
# vortex.setAgeLimit(2000.0)

#############################################################################

# Create a vortex generator:

vortex = VEE_ObjectFactory_create("vortex")
vortex.setAgeLimit(1.5);
vortex.setValueFloat("range", 1.0);
vortex.setValueFloat("magnitude", 0.5);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex)
screator.thisown = 0;

# Random direction:
vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())
screator.setRate(1.0);

scene.addNewObject(screator)

#############################################################################

# Create a tube force that keeps the flames inside:

tube = VEE_ObjectFactory_create("tube")

rot = VEE_Matrix3()

rot.rotateX(VEE_HALF_PI);
tube.setValueMatrix3("rotation", rot);
tube.setLocation3(1.0, 3.0, 0.0);

scene.addNewObject(tube)

#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)

wind = VEE_ObjectFactory_create("wind")
scene.addNewObject(wind)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings fo a couple a few parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Flame agelimit", "agelimit", p, agelimit, .5, 6.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Gravity", "gravity", g, gravity, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 300.0, 1.0, 2000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Tube magnitude", "magnitude",
                            tube, 50.0, -100.0, 100.0,
                            VEE_MAP_LINEAR);
paramView.addFloatParameter("Tube radius", "radius", tube, 1.2, 0.3, 10.0,
                            VEE_MAP_LINEAR);
paramView.addFloatParameter("Tube height", "height", tube, 6.0, 0.3, 10.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Vortex rate", "rate", screator, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex age", "type/agelimit", screator,
                            2.0, 1.0, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex range", "range", vortex, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, 3.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.5, -2, 2,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind strength", "strength", wind, 3.0, 0.0, 10,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gustiness", "gustiness", wind, 2.0, 0.0, 20,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gust frequency", "gustfreq",
                            wind, 0.35, 0.1, 20,
                            VEE_MAP_LOGARITHMIC);

#############################################################################

# Make it visible:

vq = VEE_Qt(scene)
# Make force invisible:
vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()

# The following directs the output to a file:
# vq.setFrameRate(25);
# vq.setFrameFile("Frame-%.4d.png");

paramView.addFloatParameter("Particle size", "GL/size",
                            vq, 0.7, 0.01, 3, VEE_MAP_LOGARITHMIC)

paramView.addFloatParameter("Particle alpha", "GL/alphamul",
                            vq, 1.0, 0.01, 5.0, VEE_MAP_LOGARITHMIC)

paramView.addFloatParameter("Image blending", "GL/fade",
                            vq, 0.0, 0.0, 2.0, VEE_MAP_LINEAR)

paramView.addFloatParameter("Split boxes at", "splitat",
                            vq, 30.0, 6.0, 2000.0, VEE_MAP_LOGARITHMIC)

paramView.addFloatParameter("Collapse boxes at", "collapseat",
                            vq, 20.0, 6.0, 2000.0, VEE_MAP_LOGARITHMIC)

paramView.addFloatParameter("Max recursion", "maxrecursion",
                            vq, 1.0, 0.0, 15.0, VEE_MAP_LINEAR)

paramView.bangValues()

paramView.show()

qa.execute()

