# Simple fire effect

# Usual initializations:
from vee_init import *
from vee import *
import math
qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

# Performance:
#
# "-O2": 30-35
# "-O3 -fomit-frame-pointer": 32-36
# -> heavy flags do not really help


#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_TailParticle3()
# p.resizeTail(50);
p.thisown = 0

# p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(0.7, 0.0, 0.0, 1.0)
rgbatable.appendColor(color)
color.set(0.7, 0.7, 0.3, 1.0)
rgbatable.appendColor(color)
# color.set(0.3, 0.3, 0.3, 1.0)
# rgbatable.appendColor(color)
color.set(0.0, 0.0, 0.0, 0.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)


#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;
creator.setVisible(0);

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

v1.make(-2.0, 0.0, -1.0)
v2.make(2.0, 0.5, 1.0)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)

#############################################################################

# vortex = VEE_ObjectFactory_create("vortex")
# scene.addNewObject(vortex)
# vortex.setAgeLimit(2000.0)

#############################################################################

# Create a vortex generator:

vortex = VEE_ObjectFactory_create("vortex")
vortex.setAgeLimit(1.5);
vortex.setValueFloat("area", 0.2);
vortex.setValueFloat("mass", 0.3);
vortex.setValueFloat("maxforce", 25.4);
vortex.setValueFloat("range", 1.0);
vortex.setValueFloat("minradius", 0.7);
vortex.setValueFloat("gravitate", 1.5);
vortex.setValueFloat("height", 5.0);
vortex.setValueFloat("magnitude", 3.5);
vortex.setValueVector3f("axes", 0, 0, 1);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex)
screator.thisown = 0;
screator.setVisible(0);

# Randomize location:

magnitudeMax = 27.0;
rf.set(magnitudeMax * 0.5, magnitudeMax)

vo.set("magnitude", rf.clone())
creator.appendOperator(vo.clone())

# Random direction:

randMesh = VEE_RandOnMesh2D()

randMesh.resize(6, 6);
randMesh.clear();
randMesh.setDecay(0.97)
randMesh.setThreshold(0.5)
randMesh.setValueVector3f("stretch", 8.0, 12.0, 0.0)
randMesh.setId("addlocation");

screator.appendOperator2(randMesh.clone(), "randmesh")
screator.setRate(1.0);

# Random direction:
v1.make(0.0, 0.0, -1.0)
v2.make(0.0, 0.0, 1.0)

rv.setRanges(v1, v2)

vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
scene.addNewObject(screator)

minrad = vortex.getVariableFloat("minradius", -1.0);

print "minrad is "
print minrad


#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)

# wind = VEE_ObjectFactory_create("wind")
# scene.addNewObject(wind)

#grav = VEE_ObjectFactory_create("gravitate")
#grav.setValueFloat("range", 2.0);
#grav.setValueFloat("magnitude", -200); # Push away from object
#grav.setValueFloat("maxforce", 10);
#grav.setLocation3(1.0, 4.0, 0.0);
#scene.addNewObject(grav)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings for a couple of parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Flame agelimit", "agelimit", p, 3.7, .5, 6.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame tail", "tailsize", p, 20, 4, 200,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Flame tail vel", "tailvel", p, 0.2, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame tail thresh", "threshold", p, 0.2, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Gravity", "gravity", g, 5, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 100.0, 1.0, 4000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addVector3Parameter("Flame location", "location", creator,
                              0.0, -5.0, 5.0,
                              -6, -8.0, 3.0,
                              0.0, -5.0, 5.0);

paramView.addFloatParameter("Vortex rate", "rate", screator, 11.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex rand decay", "childops/randmesh/decay",
                            screator, .978, 0.5, 1.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Vortex range", "range", vortex, 1.3, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex min rad", "minradius", vortex, 0.8, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex gravitate", "gravitate", vortex, 1.4, -1, 3.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Vortex age", "agelimit", vortex, 1.7, 0.5, 10.0,
                            VEE_MAP_LOGARITHMIC);

# paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, 3.0, 0.1, 20.0,
#                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.6, -2, 2,
                            VEE_MAP_LINEAR);

# paramView.addFloatParameter("Wind strength", "strength", wind, 3.0, 0.0, 10,
#                             VEE_MAP_LINEAR);

# paramView.addFloatParameter("Wind gustiness", "gustiness", wind, 3.0, 0.0, 20,
#                             VEE_MAP_LINEAR);

# paramView.addFloatParameter("Wind gust frequency", "gustfreq",
#                             wind, 0.35, 0.1, 20,
#                             VEE_MAP_LOGARITHMIC);


#############################################################################

# Make it visible:

vq = VEE_Qt(scene)

vq.setValueFloat("GL/size", 0.4);
vq.setFrameFile("frames/Vee-2-%.5d.jpg")

# Make force invisible:
vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()

paramView.bangValues()

paramView.show()

print vq.renderer().subTextureCount()

qa.execute()

