2# Simple fire effect

# Usual initializations:
from vee_init import *
from vee import *
import math
qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

agelimit = 2.0
gravity = 5.0

# Performance on pyry (rh 7.1)
# Points: 430
# lines: 430
# Quads (medium, stars): 340
# Quads (large): 250

# Complex (about 1200 particles per second) 75fps

# SPEEDS:
#
# RH 7.1: 320-370
#

#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_Particle2()
p.thisown = 0

# p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(1.0, 0.3, 0.0, 0.5, 0.9)
rgbatable.appendColor(color)
color.set(1.0, 0.5, 0.2, 0.5, 0.8)
rgbatable.appendColor(color)
color.set(0.7, 0.5, 0.2, 0.5, 0.5)
rgbatable.appendColor(color)
color.set(0.4, 0.4, 0.4, 0.7, 0.2)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.4, 0.0, 0.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)

sizecurve = VEE_ValueCurve2();
sizecurve.setId("size");
sizecurve.appendPoint(0.3, 0.0);
sizecurve.appendPoint(0.3, 0.2);
sizecurve.appendPoint(0.4, 0.6);
sizecurve.appendPoint(1.0, 1.0);
sizecurve.setScaling(VEE_TS_AGE_MATCH);

p.appendSelfOperator(sizecurve.clone());

#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;
creator.setVisible(0)

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

v1.make(-2.0, 0.0, -2.0)
v2.make(2.0, 0.5, 2.0)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)


#############################################################################

# Create a random force:

randforce = VEE_ObjectFactory_create("randomforce")

scene.addNewObject(randforce)

#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)

wind = VEE_ObjectFactory_create("wind")
scene.addNewObject(wind)

#grav = VEE_ObjectFactory_create("gravitate")
#grav.setValueFloat("range", 2.0);
#grav.setValueFloat("magnitude", -200); # Push away from object
#grav.setValueFloat("maxforce", 10);
#grav.setLocation3(1.0, 4.0, 0.0);
#scene.addNewObject(grav)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings fo a couple a few parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Gravity", "gravity", g, gravity, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame agelimit", "agelimit", p, 3, .5, 6.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 300.0, 1.0, 4000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addVector3Parameter("Noise magnitude", "magnitude", randforce,
                              10.0, 0.0, 40.0,
                              0.0, 0.0, 40.0,
                              0.0, 0.0, 40.0);

paramView.addFloatParameter("Noise octaves", "octaves", randforce, 1, 1.0, 10.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Noise attenuation", "attenuation", randforce, 0.7, 0.0, 1.0,
                            VEE_MAP_LINEAR);

paramView.addVector3Parameter("Noise scale", "scale", randforce, 
                              2.0, 0.0, 10.0,
                              0.3, 0.0, 10.0,
                              0.0, 0.0, 10.0);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.5, -2, 2,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind strength", "strength", wind, 0.0, 0.0, 10,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gustiness", "gustiness", wind, 3.0, 0.0, 20,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gust frequency", "gustfreq",
                            wind, 0.35, 0.1, 20,
                            VEE_MAP_LOGARITHMIC);


#############################################################################

# Make it visible:

vq = VEE_Qt(scene)
# Make force invisible:
vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()

paramView.bangValues()

paramView.show()

print vq.renderer().subTextureCount()

qa.execute()

