# Simple fire effect

# Usual initializations:
from vee_init import *
from vee import *
import math
qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_Particle()
p.thisown = 0

# p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(0.7, 0.0, 0.0, 1.0)
rgbatable.appendColor(color)
color.set(0.7, 0.7, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.0, 0.0, 0.0, 0.1)
rgbatable.appendColor(color)
color.set(0.0, 0.0, 0.0, 0.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)


#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

v1.make(-1.0, -1.0, -1.0)
v2.make(1.0, 1.0, 1.0)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())
vo.set("area", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)

#############################################################################

# Now we create a blow object that pushes the stuff to given direction

blow = VEE_ObjectFactory_create("blow")

blow.setLocation3(-1.5, 0.0, 0.0);
blow.setValueFloat("range", 10);
blow.setValueFloat("magnitude", 20);
blow.setValueVector3f("direction", 1.0, 0.0, 0.0);

scene.addNewObject(blow)

#############################################################################


vortex = VEE_ObjectFactory_create("vortex")
vortex.setAgeLimit(1.5);
vortex.setValueFloat("range", 1.7);
vortex.setValueFloat("magnitude", 16.5);
vortex.setValueFloat("maxforce", 6.0);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex.clone())

vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())

gradient = VEE_OperatorForceGradient()
gradient.appendPoint(0.01, 0.0, 0);
gradient.appendPoint(0.1, 0.1, 0);
gradient.appendPoint(0.2, 1.0, 0);
gradient.appendPoint(0.3, 2.0, 0);
gradient.setId("rate", 0)

screator.appendSelfOperator2(gradient.clone(), "gradient")

randomvects = VEE_VolumeBox();
randomvects.setAxes(3, 1.5, 1.5)

v1.make(2, 0, 0)

targets = VEE_ParamTargets()

n = 20;
for i in range(0,n):
	loc = randomvects.newRandomPoint()
	loc = loc + v1
	screatorTmp = screator.clone();
	screatorTmp.setLocation(loc);
	scene.addNewObject(screatorTmp);
	targets.append(screatorTmp, "selfops/gradient/scale")

#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)


tube = VEE_ObjectFactory_create("tube")

rot = VEE_Matrix3()

rot.rotateX(VEE_HALF_PI);
tube.setValueMatrix3("rotation", rot);
tube.setLocation3(6.0, 0.0, 0.0);

scene.addNewObject(tube)


# wind = VEE_ObjectFactory_create("wind")
# scene.addNewObject(wind)

#grav = VEE_ObjectFactory_create("gravitate")
#grav.setValueFloat("range", 2.0);
#grav.setValueFloat("magnitude", -200); # Push away from object
#grav.setValueFloat("maxforce", 10);
#grav.setLocation3(1.0, 4.0, 0.0);
#scene.addNewObject(grav)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings to a couple of parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Flame agelimit", "agelimit", p, 3.0, .5, 6.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Gravity", "gravity", g, 0.0, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 300.0, 1.0, 4000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Blow magnitude", "magnitude", blow,
			    20.0, 1.0, 200.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Tube magnitude", "magnitude",
                            tube, 50.0, -100.0, 100.0,
                            VEE_MAP_LINEAR);
paramView.addFloatParameter("Tube radius", "radius", tube, 1.2, 0.3, 10.0,
                            VEE_MAP_LINEAR);
paramView.addFloatParameter("Tube height", "height", tube, 6.0, 0.3, 10.0,
                            VEE_MAP_LINEAR);

slider = paramView.addFloatParameter("Vortex rate scale",
				     "selfops/gradient/scale", screator,
				     0.5, 0.0, 1.0,
				     VEE_MAP_LINEAR);


slider.addTargets(targets)

# .addFloatParameter("Vortex rate", "rate", screator, 2.0, 0.1, 20.0,
#                             VEE_MAP_LOGARITHMIC);

# paramView.addFloatParameter("Vortex range", "range", vortex, 2.0, 0.1, 20.0,
#                             VEE_MAP_LOGARITHMIC);

# paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, 3.0, 0.1, 20.0,
#                             VEE_MAP_LOGARITHMIC);



#############################################################################

# Make it visible:

vq = VEE_Qt(scene)
# Make force invisible:
vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()

vq.setFrameFile("frames/frame-%.5d.jpeg");

paramView.bangValues()

paramView.show()

print vq.renderer().subTextureCount()

qa.execute()

