# Simple fire effect

# Usual initializations:
from vee_init import *
from vee import *
import math
qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

agelimit = 10.0
gravity = 5.0

# SPEEDS
# RH7.1: 170-188

#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_Particle()
p.thisown = 0

p.setAgeLimit(100000.0)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(0.3, 0.3, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.7, 0.7, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.0, 1.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)

#############################################################################

# Now for the star creator
creator =  VEE_Emitter(p)
creator.thisown = 0;

v1 = VEE_Vector3()
v2 = VEE_Vector3()

v1.make(-0.8, -5.0, -0.5)
v2.make( 0.8,  5.0, 0.5)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

# vo.set("addvelocity", rv.clone())

# creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.95, 1.0)

vo.set("area", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)

n = 4000

for i in range(0, n):
    creator.setValueInt("trigger", 1)

creator.setRate(50)
creator.setAgeLimit(10.0)


#############################################################################

# The central vortex in the galaxy:

vortex = VEE_ObjectFactory_create("vortex")
vortex.setValueFloat("range", 10.0);
vortex.setValueFloat("faderange", 0.1);
vortex.setValueFloat("magnitude", 0.01);
vortex.setValueFloat("epsilon", 0.05);
vortex.setVelocityUpdate(0)
vortex.setAgeLimit(1000000)
v1.make(0.0, 0.0, 2.0)
vortex.setValueVector3("axes", v1)

scene.addNewObject(vortex)

# Create the vortex generator:

vortex2 = VEE_ObjectFactory_create("vortex")
vortex2.setAgeLimit(14.5);
vortex2.setValueFloat("magnitude", 0.1);
vortex2.setValueFloat("range", 1.0);
vortex2.setVelocityUpdate(1)

v1.make(0.0, 0.0, -2.0)
vortex2.setValueVector3("axes", v1)

screator = VEE_Emitter(vortex2)
screator.thisown = 0;

# Randomize location:
v1.make( 2.6, -5.0, 0.0)
v2.make( 3.6,  5.0, 0.1)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())
screator.setRate(1.0);

scene.addNewObject(screator)

#############################################################################

# Create a few forces:

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)

#############################################################################

# Make GUI mappings fo a couple a few parameters:

paramView = VEE_ValueWindow()

#paramView.addFloatParameter("Flame agelimit", "agelimit", p, agelimit,
#			    .5, 20.0,
#                            VEE_MAP_LINEAR);

#paramView.addFloatParameter("Flame rate", "rate", creator, 300.0, 1.0, 4000.0,
#                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex rate", "rate", screator, .14, 0.01, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex range", "range", vortex, 6.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, .3, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.2, -2, 2,
                            VEE_MAP_LINEAR);


#############################################################################

# Make it visible:

vq = VEE_Qt(scene)
# Make force invisible:
vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()
# vq.setValueFloat("GL/size", 0.9);

vq.setValueFloat("GL/size", 0.29);
vq.setValueFloat("GL/lengthscale", 2.0);
# vq.setValueFloat("GL/alphamul", 0.4);

vq.setFrameFile("frames/galaxy-%.6d.jpeg");
# vq.fadeMore();

#vq.setValueInt("fileoversample", 4)
#vq.setValueFloat("speed", 2)

paramView.addFloatParameter("Particle size", "GL/size",
                            vq, 0.4, 0.01, 3, VEE_MAP_LOGARITHMIC)

paramView.addFloatParameter("Particle alpha", "GL/alphamul",
                            vq, 1.0, 0.01, 1.0, VEE_MAP_LINEAR)


paramView.addFloatParameter("Texture", "GL/subtexture",
                            vq, 13,
			    0, vq.renderer().subTextureCount() - 1,
			    VEE_MAP_LINEAR)

vq.addKeyPoint2(-0.000, -5.581, 5.564, 3.142, 0.700);
vq.addKeyPoint2(-0.000, -6.395, 2.568, 3.142, 0.900)

paramView.bangValues()

paramView.show()

print vq.renderer().subTextureCount()

vq.toggleCalculusUpdate()

qa.execute()

