# Simple fire effect

# Usual initializations:
from vee_init import *
from vee import *

qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()


#############################################################################

# Initialize useful general-purpose variables

v1 = VEE_Vector3()
v2 = VEE_Vector3()

m1 = VEE_Matrix3()

#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_Polygon_createNewMesh(6, 6, 6.0, 3.0, 2.0)
p.thisown = 0
p.setMaterial(0)

rgbatable = VEE_RgbaTable()
# rgbatable.thisown = 0
color = VEE_Rgba()

color.set(0.7, 0.0, 0.0, 0.6)
rgbatable.appendColor(color)
color.set(0.7, 0.7, 0.3, 0.4)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.3, 0.4)
rgbatable.appendColor(color)
color.set(0.0, 0.0, 0.0, 1.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)

m1.rotateZ(VEE_HALF_PI)
p.setValueMatrix3("transform", m1)

v1.clear();
p.setValueVector3("location", v1)

#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;
creator.setVisible(0)

r = p.setValue("looks", valueLooks)


v1.make(-2.0, 0.0, -2.0)
v2.make(2.0, 0.5, 2.0)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator2(vo.clone(), "addlocation")

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)

rr = VEE_ValueRandomRotation()
vo.set("transform", rr.clone())
creator.appendOperator(vo.clone())

dtop = VEE_OperatorDt()
dtop.tthisown = 0
dtop.set(2.0, 1, 1)
dtop.setMaxStep(0.3)
creator.appendOperator(dtop)


#############################################################################

# Create a vortex generator:

vortex = VEE_ObjectFactory_create("vortex")
# vortex.thisown = 0
vortex.setAgeLimit(1.5);
# vortex.setValueFloat("range", 1.0);
# vortex.setValueFloat("magnitude", 0.5);
# vortex.setValueFloat("maxforce", 10.5);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex)
screator.setVisible(0)

vortexrange = VEE_ValueRandomFloat()
vortexrange.thisown = 0
vortexrangeop = VEE_ValueOperator()
vortexrangeop.set("range", vortexrange);
vortexrangeop.thisown = 0

screator.appendOperator(vortexrangeop)

# Random direction:
vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())
screator.setRate(1.0);


scene.addNewObject(screator)

#############################################################################

# Create local wind gusts (emitted from the center of the fire)

wind2 = VEE_ObjectFactory_create("wind")

v1.make(7.0, 4.0, 0.0);
wind2.setValueVector3("velocity", v1)
wind2.setValueVector3("direction", v1)
wind2.setValueVector3("location", v1)
wind2.setValueFloat("strength", 20.0)
wind2.setAgeLimit(3.0);

vsphere = VEE_VolumeSphere()
vsphere.set(2.0, 3.0);
vvolume = VEE_ValueVolume()
vvolume.setObject(vsphere.clone())
wind2.setValue("volume", vvolume);

wcreator = VEE_Emitter(wind2)
wcreator.setVisible(0)
wcreator.setRate(0.5)
wcreator.setInheritVelocity(0)

v1.make(-4.0, 2.0, -5)
v2.make(10.0, 10.5, 5.0)
rv.setRanges(v1, v2)

vo.set("addvelocity", rv.clone())
wcreator.appendOperator(vo.clone())
vo.set("direction", rv.clone())
wcreator.appendOperator(vo.clone())

scene.addNewObject(wcreator)

#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)

wind = VEE_ObjectFactory_create("wind")
scene.addNewObject(wind)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings fo a couple a few parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Flame agelimit", "agelimit", p, 3.0, .5, 6.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Gravity", "gravity", g, 5.0, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 10.0, .5, 50.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addVector3Parameter("Flame location", "location", creator,
                              0.0, -5.0, 5.0,
                              0.0, -5.0, 5.0,
                              0.0, -5.0, 5.0);

paramView.addVector3Parameter("Flame randmin", "childops/addlocation/min",
                              creator,
                              -1.5, -5.0, 5.0,
                              0.0, -5.0, 5.0,
                              -1.5, -5.0, 5.0);

paramView.addVector3Parameter("Flame randmax", "childops/addlocation/max",
                              creator,
                              1.5, -5.0, 5.0,
                              0.0, -5.0, 5.0,
                              1.5, -5.0, 5.0);

paramView.addFloatParameter("Vortex rate", "rate", screator, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex range min", "min",
                            vortexrangeop, 1.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex range max", "max",
                            vortexrangeop, 2.5, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex magnitude", "magnitude",
                            vortex, 3.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex max force", "maxforce",
                            vortex, 7.0, 0.1, 100.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.5, -2, 2,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind strength", "strength", wind, 3.0, 0.0, 10,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gustiness", "gustiness", wind, 5.0, 0.0, 20,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gust frequency", "gustfreq",
                            wind, 0.35, 0.1, 20,
                            VEE_MAP_LOGARITHMIC);


#############################################################################

# Make it visible:

vq = VEE_Qt(scene)
# Make force invisible:
vq.toggleForceMode();
qa.setMainWidget(vq)


paramView.addFloatParameter("Particle size", "GL/size",
                            vq, 0.7, 0.01, 3, VEE_MAP_LOGARITHMIC)

paramView.addFloatParameter("Particle alpha", "GL/alphamul",
                            vq, 1.0, 0.01, 1.0, VEE_MAP_LINEAR)

paramView.addFloatParameter("Field of view", "GL/fov",
                            vq, 70.0, 10, 175, VEE_MAP_LINEAR)

paramView.bangValues()

paramView.move(600, 100)

vq.move(100, 100)

vq.raiseWidget()

paramView.show()
vq.show()


qa.execute()

