
# Usual initializations:

from vee_init import *
from vee import *
import math

qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

agelimit = 5.0
gravity = 5.0

# object = "cat.lwo";
object = "shaft.lwo";
# object = "cupbase.lwo";
# object = "/usr/share/Performer/data/teapot.dxf";
objectscale = 0.6;

# objectLwo = object + ".lwo"
objectField = object + ".vfi"

model = Solar_Model();
Solar_Loaders_load(model, object);

rebuildField = 1

if rebuildField:
    vgrid = VEE_VectorGrid();
    vgrid.createFromModel(model, 40, 4, 2, -1);
    # vgrid.createFromModel2(model, 40, 4, 2);
    vgrid.writeGridToFile(objectField)

    saveVfa = 0
    
    if saveVfa:
        vgrid.writeAsciiFile(object + ".vfa", vgrid.layers() / 2);

# res = VEE_VectorGrid_fromModelToFile(objectField, object, 70, 3, 3, 1);

rot = VEE_Matrix3();
rot.identity()

rot.rotateY(-VEE_HALF_PI)
# rot.rotateZ(1.0)

#############################################################################

vgrid = VEE_VectorGrid();

res = vgrid.readGridFromFile(objectField)

gridforce = VEE_GridForce();

gridforce.setGrid(vgrid)
gridforce.setValueFloat("magnitude", 20.0);
gridforce.setValueFloat("scalemul", objectscale);

objloc = VEE_Vector3();

objloc.make(0.0, 0.0, 0.0);

gridforce.setLocation(objloc);
gridforce.setValueMatrix3("rotation", rot);

scene.addNewObject(gridforce)


#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_TailParticle()
p.thisown = 0

# p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(0.7, 0.0, 0.0, 1.0)
rgbatable.appendColor(color)
color.set(0.7, 0.7, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.0, 0.0, 0.0, 1.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)


#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

range = 0.35

v1.make(-range, -0.5 * range , -range)
v2.make( range,  0.5 * range,  range)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)

#############################################################################

# Build a second particle with ugly colors, lifespan etc:

p2 = VEE_Particle()
p2.thisown = 0
p2.setAgeLimit(2);

# p.setAgeLimit(agelimit)

# rgbatable2 = VEE_RgbaTable()
# color2 = VEE_Rgba()

# color2.set(0.0, 0.0, 0.5, 1.0)
# rgbatable2.appendColor(color2)
#color2.set(0.2, 0.0, 0.7, 1.0)
#rgbatable2.appendColor(color2)
#color2.set(0.3, 0.0, 0.3, 1.0)
#rgbatable2.appendColor(color2)
#color2.set(0.0, 0.0, 0.0, 1.0)
#rgbatable2.appendColor(color2)

looks2 = VEE_LooksColorTable(rgbatable)
looks2.thisown = 0

valueLooks2 = VEE_ValueLooks()
valueLooks2.setLooks(looks2)

p2.setValue("looks", valueLooks2)



#############################################################################

# vortex = VEE_ObjectFactory_create("vortex")
# scene.addNewObject(vortex)
# vortex.setAgeLimit(2000.0)

#############################################################################

# Create a vortex generator:

vortex = VEE_ObjectFactory_create("vortex")
vortex.setAgeLimit(1.5);
vortex.setValueFloat("range", 1.0);
vortex.setValueFloat("magnitude", 0.5);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex)
screator.thisown = 0;

# Random direction:
vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())
screator.setRate(1.0);

scene.addNewObject(screator)


#############################################################################

# Create a few forces:

w = VEE_ObjectFactory_create("wind")
w.setValueVector3f("direction", -1, 0, 0);
scene.addNewObject(w)

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)


#grav = VEE_ObjectFactory_create("gravitate")
#grav.setValueFloat("range", 2.0);
#grav.setValueFloat("magnitude", -200); # Push away from object
#grav.setValueFloat("maxforce", 10);
#grav.setLocation3(1.0, 4.0, 0.0);
#scene.addNewObject(grav)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings fo a couple a few parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Wind speed", "strength", w, 5, 0, 20.0,
                            VEE_MAP_LINEAR);



paramView.addFloatParameter("Flame agelimit", "agelimit", p, agelimit, .5, 10.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid Bounce distance",
                            "bouncedistance", gridforce,
                            0.3, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid flow distance",
                            "flowdistance", gridforce,
                            1.0, 0.0, 4.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid drag distance",
                            "dragdistance", gridforce,
                            0.2, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid magnitude", "magnitude", gridforce,
                            400.0, .5, 1000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Grid bounciness", "bounce", gridforce,
                            0.0, 0.0, 1.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid drag", "drag", gridforce,
                            0.0, 0.0, 7.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid drag power", "dragpower", gridforce,
                            2.0, 0.0, 3.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid tangent force", "tangentforce", gridforce,
                            9.0, -100.0, 100.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid tangent grab", "tangentgrab", gridforce,
                            1.6, -100.0, 100.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame tail", "tailsize", p, 20, 4, 200,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Flame tail thresh", "threshold", p, 0.2, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 50.0, 0.0, 200.0,
                            VEE_MAP_LINEAR);

paramView.addVector3Parameter("Flame location", "location", creator,
                              4.0, -7.0, 7.0,
                              0.0, -4.0, 4.0,
                              0.0, -4.0, 4.0);

paramView.addFloatParameter("Vortex rate", "rate", screator, 0.0, 0.0, 3.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Vortex range", "range", vortex, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, 3.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex maxforce", "maxforce", vortex, 10.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.2, -2, 6,
                            VEE_MAP_LINEAR);

#############################################################################

# keypoint location = [0.467 0.300 9.545] azi = 3.142, ele = 0.000

# Make it visible:

vq = VEE_Qt(scene)
# Make force invisible:
q2 = VEE_TestQt(vq, scene, "", 640, 480, 0, vq)
q2.setMtlFile("vee_materials.mtl");
vq.setGlWidget(q2);

vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()
# vq.setRenderOrder("1");
vq.setValueFloat("GL/size", 0.05);
vq.setValueInt("GL/subtexture", 5);
vq.setFrameFile("frames/shaft-%.5d.jpg")

kp = vq.addKeyPoint2(0.293, 0.000, 6.793, 3.142, 0.000)
vq.selectKeyPoint2(kp);

model = Solar_Model();

ok = Solar_Loaders_load(model, object);

model.centralize();
model.scale(objectscale);
solarobj = Solar_Object()

solarobj.setModel(model)
solarobj.setLocation(objloc)
solarobj.setRotation(rot)

q2.addModel(model);
q2.addObject(solarobj);

paramView.bangValues()

paramView.show()

qa.execute()

