/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#include <Loaders.hpp>

#include <Error.hpp>
#include <LwoConvert.hpp>

#ifdef HAVE_PERFORMER
#include <PfConvert.hpp>
#endif

#include <string.h>

namespace Solar {

  bool Loaders::load(Model *m, const char *filename)
  {
    const char * fname = "Loaders::load";

    int l = strlen(filename);

    bool tryLwo = false;

    if(l > 4)
      if(strcmp(filename + l - 4, ".lwo") == 0)
	tryLwo = true;

    bool ok = false;

    if(tryLwo) {
      trace("%s # Using LWO loader to load \"%s\"", fname, filename);
      ok = LwoConvert::load(m, filename);
    }
#ifdef HAVE_PERFORMER
    else {
      trace("%s # Using Performer loader to load \"%s\"", fname, filename);

      if(!pfIsInited())
	pfInit();

      ok = PfConvert::load(m, filename);
    }
#endif

    return ok;
  }
}
